----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:02:18 07/23/2014 
-- Design Name: 
-- Module Name:    CONTROL_NEXUS - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;


use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


-- This module combines everything between the Zarlink and the FIFO from the RHA 
entity CONTROL_NEXUS is
    Port (

		-- Connection to the Zarlink 
		SPI_ZL_CS 					: out STD_LOGIC;
		SPI_SDI 					: out STD_LOGIC;
		SPI_SDO 					: in  STD_LOGIC;
		SPI_CLK 					: out STD_LOGIC;		

		-- RHA 
		ResetRHAandFIFO				: out STD_LOGIC;

		-- Parameters for the header
		ChannelMask     			: out std_logic_vector(127 downto 0);
		ADCResolution   			: out std_logic_vector(3 downto 0);
		
		-- We also need a variable clock that is looking on this SampleRate
		-- 4MHz = 10KHz 
		SampleRate      			: out std_logic_vector(7 downto 0);
		
		Filter          			: out std_logic_vector(3 downto 0);
		DeviceID        			: out std_logic_vector(7 downto 0);		
		
		-- Bitmask for the data selection from the 16 bit samples.
		BITMASK_SAMPLE				: out std_logic_vector(15 downto 0);			 
		Clk_Reduced					: out STD_LOGIC;

		-- Connection to the RHA DATA FIFO...
		-- Data 
		FIFO_IN_DATA				: in  STD_LOGIC_VECTOR(7 downto 0);	
		-- At least 14 x 8 bit are availabe if this is one...
		FIFO_IN_VALID				: in	STD_LOGIC;
		-- Need new data 
		FIFO_IN_READENABLE			: out STD_LOGIC;
		
		-- RHA Testmodule 
		TestMode					: out STD_LOGIC;

		DEBUG_STRING_ORGANIZER		: out std_logic_vector (3 downto 0);

		-- The usual ports 
		Reset 						: in  STD_LOGIC;	
		Clk_FAST					: in  STD_LOGIC;			
		Clk 						: in  STD_LOGIC
	);
end CONTROL_NEXUS;

architecture Behavioral of CONTROL_NEXUS is

		signal Zarlink_WriteEn							: STD_LOGIC := '0'; 
		signal Zarlink_Adress							: STD_LOGIC_VECTOR(7 downto 0) := (others => '0'); 		
		signal Zarlink_Data_Out							: STD_LOGIC_VECTOR(7 downto 0) := (others => '0'); 
		signal Zarlink_RequireValid_Out				: STD_LOGIC := '0'; 
		signal Zarlink_Data_In							: STD_LOGIC_VECTOR(7 downto 0) := (others => '0'); 
		signal Zarlink_Valid_In							: STD_LOGIC := '0';
		signal Zarlink_BlockMode						: STD_LOGIC := '0'; 
		signal Zarlink_Wait								: STD_LOGIC := '0'; 

		-- ZarlinkConnectionInit 
		signal ZarlinkConnectionInit_DO				: STD_LOGIC := '0';

		-- CheckConnection
		signal CheckConnection_CHECK					: STD_LOGIC := '0';
		signal CheckConnection_OK						: STD_LOGIC := '0';

		-- CheckIngoingBuffer
		signal CheckIngoingBuffer_CHECK				: STD_LOGIC := '0';
		signal CheckIngoingBuffer_OK					: STD_LOGIC := '0';
		signal CheckIngoingBuffer_SIZE				: STD_LOGIC_VECTOR (6 downto 0) := (others => '0');

		-- CheckOutgoingBuffer
		signal CheckOutgoingBuffer_CHECK				: STD_LOGIC := '0';
		signal CheckOutgoingBuffer_OK					: STD_LOGIC := '0';
		signal CheckOutgoingBuffer_SIZE				: STD_LOGIC_VECTOR (6 downto 0) := (others => '0');

		-- GetCommand
		signal GetCommand_GET							: STD_LOGIC := '0';
		signal GetCommand_DATA							: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal GetCommand_VALID							: STD_LOGIC := '0';

		-- Write data to Zarlink
		signal Zarlink_FIFO_WAIT						: STD_LOGIC := '0';
		signal Zarlink_FIFO_DATA						: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal Zarlink_FIFO_FILLED						: STD_LOGIC := '0';
		signal Zarlink_FIFO_WRITE						: STD_LOGIC := '0';
		
		signal FIFO_IN_READENABLE_INTERNAL			: STD_LOGIC := '0';
		
		signal ResetRHAandFIFO_INTERNAL				: STD_LOGIC := '0';
		
		signal Clock_4MHz 								: STD_LOGIC := '0';
		signal WaitSignal_ProcessOngoing			: STD_LOGIC := '0';
		signal ProcessOngoing 						: STD_LOGIC := '0';

begin
	-- Export the internal buffered signals 
	ResetRHAandFIFO <= ResetRHAandFIFO_INTERNAL;

	-- Connection to the RHA DATA FIFO...
	-- Data 
	Zarlink_FIFO_DATA <=  FIFO_IN_DATA;
	-- At least 14 x 8 bit are availabe if this is one...
	Zarlink_FIFO_FILLED <= FIFO_IN_VALID;
	-- Need new data 
	FIFO_IN_READENABLE <= Zarlink_RequireValid_Out;

	-- This module is responsible for interfacing the Zarlink hardware via SPI


	-- This module analyzes the incomming data from the Zarlink and reacts to these commands 
	Inst_COMMAND_RECEIVER: entity work.COMMAND_RECEIVER 
	PORT MAP(
		ChannelMask => ChannelMask,
		ADCResolution => ADCResolution,
		SampleRate => SampleRate,
		Filter => Filter,
		DeviceID => DeviceID,
		BITMASK_SAMPLE => BITMASK_SAMPLE,
		FIFO_OUT_DATA => GetCommand_DATA,
		FIFO_OUT_VALID => GetCommand_VALID,
		Clk_Reduced => Clk_Reduced,
		ResetRHAandFIFO => ResetRHAandFIFO_INTERNAL,
		Clock_4MHz => Clock_4MHz,
		TestMode => TestMode,
		Clk_FAST  => Clk_FAST,
		Clk => Clk,
		Reset => Reset
	);

-------------------------------------------------------------------

-- This module is responsible for interfacing the Zarlink hardware via SPI
	Inst_Zarlink_SPI_Module: entity work.Zarlink_SPI_Module 
	PORT MAP(
		Zarlink_WriteEn => Zarlink_WriteEn,
		Zarlink_Adress => Zarlink_Adress,
		Zarlink_Data_Out => Zarlink_Data_Out,
		Zarlink_Data_In => Zarlink_Data_In,
		Zarlink_Valid_In => Zarlink_Valid_In,
		Zarlink_RequireValid_Out => Zarlink_RequireValid_Out,
		Zarlink_BlockMode => Zarlink_BlockMode,
		Zarlink_Wait => Zarlink_Wait,
		SPI_ZL_CS => SPI_ZL_CS,
		SPI_SDI => SPI_SDI,
		SPI_SDO => SPI_SDO,
		SPI_CLK => SPI_CLK,
		Reset => Reset,
		Clk => Clock_4MHz
	);

	-- This module manages all the commands (connection initialization process, check connection status, read commands, 
	-- copy data to the Zarlink, check the rx/tx buffer) concerning the Zarlink 
	Inst_ZARLINK_CONNECT_INIT: entity work.ZARLINK_CONNECT_INIT 
	PORT MAP(
		ZarlinkConnectionInit_DO => ZarlinkConnectionInit_DO,
		CheckConnection_CHECK => CheckConnection_CHECK,
		CheckConnection_OK => CheckConnection_OK,
		CheckIngoingBuffer_CHECK => CheckIngoingBuffer_CHECK,
		CheckIngoingBuffer_OK => CheckIngoingBuffer_OK,
		CheckIngoingBuffer_SIZE => CheckIngoingBuffer_SIZE,
		CheckOutgoingBuffer_CHECK => CheckOutgoingBuffer_CHECK,
		CheckOutgoingBuffer_OK => CheckOutgoingBuffer_OK,
		CheckOutgoingBuffer_SIZE => CheckOutgoingBuffer_SIZE,
		GetCommand_GET => GetCommand_GET,
		GetCommand_DATA => GetCommand_DATA,
		GetCommand_VALID => GetCommand_VALID,
		Zarlink_FIFO_WRITE => Zarlink_FIFO_WRITE,
		Zarlink_FIFO_DATA => Zarlink_FIFO_DATA,
		Zarlink_WriteEn => Zarlink_WriteEn,
		Zarlink_Adress => Zarlink_Adress,
		Zarlink_Data_Out => Zarlink_Data_Out,
		Zarlink_Data_In => Zarlink_Data_In,
		Zarlink_Valid_In => Zarlink_Valid_In,
		Zarlink_BlockMode => Zarlink_BlockMode,
		Zarlink_Wait => Zarlink_Wait,
		WaitSignal_ProcessOngoing => WaitSignal_ProcessOngoing,
		Reset => Reset,
		Clk => Clk
	);	

	Inst_ORGANIZER: entity work.ORGANIZER 
	PORT MAP(
		CheckConnection_CHECK => CheckConnection_CHECK,
		CheckConnection_OK => CheckConnection_OK,
		ZarlinkConnectionInit_DO => ZarlinkConnectionInit_DO,
		CheckIngoingBuffer_CHECK => CheckIngoingBuffer_CHECK,
		CheckIngoingBuffer_OK => CheckIngoingBuffer_OK,
		CheckIngoingBuffer_SIZE => CheckIngoingBuffer_SIZE,
		CheckOutgoingBuffer_CHECK => CheckOutgoingBuffer_CHECK,
		CheckOutgoingBuffer_OK => CheckOutgoingBuffer_OK,
		CheckOutgoingBuffer_SIZE => CheckOutgoingBuffer_SIZE,
		Zarlink_FIFO_FILLED => Zarlink_FIFO_FILLED,
		Zarlink_FIFO_WRITE => Zarlink_FIFO_WRITE,
		GetCommand_GET => GetCommand_GET,
		ProcessOngoing => ProcessOngoing,
		DEBUG_CheckOutgoingBuffer => open,
--		DEBUG_STRING => DEBUG_STRING_ORGANIZER,
		Clk => Clk,
		Reset => Reset
	);


	-- This module contains a FSM which organizes the behaviour of the implant. 
	ProcessOngoing <= (WaitSignal_ProcessOngoing);


end architecture Behavioral;

