----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:47:04 07/11/2014 
-- Design Name: 
-- Module Name:    RHA_DATA_CHECKER - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

-- This module contains all information about how a 
-- correct data string from a RHA should look like.
-- It checks it and responds with an error if
-- something is wrong. Error types are:
-- Parity error &
-- Pattern error (a reoccuring pattern) & 
-- and if the address string corresponds 
-- to the actual channel number 

entity RHA_DATA_CHECKER is
    Port (
		DATAOUT_DATA			: in  STD_LOGIC_VECTOR(15 downto 0);
		DATAOUT_ADDRESS		: in  STD_LOGIC_VECTOR(4 downto 0);
		DATAOUT_PATTERN		: in  STD_LOGIC_VECTOR(2 downto 0);
		DATAOUT_PARITY			: in  STD_LOGIC;

		DATAOUT_ADDRESS_COMP		: in  STD_LOGIC_VECTOR(4 downto 0);

		ERROR_PARITY			: out STD_LOGIC;
		ERROR_PATTERN			: out STD_LOGIC;
		ERROR_ADDRESS			: out STD_LOGIC
	 );
end RHA_DATA_CHECKER;

architecture Behavioral of RHA_DATA_CHECKER is

begin
	-- Check if the 110 pattern is correct
	PatternCheck : process (DATAOUT_PATTERN) 
	begin
		if DATAOUT_PATTERN = "110" then
			ERROR_PATTERN <= '0';
		else
			ERROR_PATTERN <= '1';
		end if;
	end process PatternCheck;

	-- Check if the address fits with the channel number
	PatternAddress : process (DATAOUT_ADDRESS, DATAOUT_ADDRESS_COMP)
	begin
		if DATAOUT_ADDRESS = DATAOUT_ADDRESS_COMP then
			ERROR_ADDRESS <= '0';
		else
			ERROR_ADDRESS <= '1';
		end if;
	end process PatternAddress;

	-- Check if the parity of data because we can...
	PatternParity : process (DATAOUT_PARITY, DATAOUT_DATA)
	begin
	
		if (DATAOUT_PARITY = NOT
			 ( DATAOUT_DATA(15) XOR DATAOUT_DATA(14) XOR DATAOUT_DATA(13) XOR DATAOUT_DATA(12)
		  XOR DATAOUT_DATA(11) XOR DATAOUT_DATA(10) XOR DATAOUT_DATA( 9) XOR DATAOUT_DATA( 8)
		  XOR DATAOUT_DATA( 7) XOR DATAOUT_DATA( 6) XOR DATAOUT_DATA( 5) XOR DATAOUT_DATA( 4)
		  XOR DATAOUT_DATA( 3) XOR DATAOUT_DATA( 2) XOR DATAOUT_DATA( 1) XOR DATAOUT_DATA( 0) )) then
			ERROR_PARITY <= '0';
		else
			ERROR_PARITY <= '1';
		end if;
	
	end process PatternParity;



end Behavioral;

