----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:47:31 07/11/2014 
-- Design Name: 
-- Module Name:    RHA_DATA_COLLECTOR - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

-- This PHY-module contains a state machine which collects the data directly 
-- from the RHA and deserializes it. 

entity RHA_DATA_COLLECTOR is
    Port ( 
		-- Connection to the RHA
		-- For the data stream
		RHA_ADC_out 			: in  STD_LOGIC;
		-- Where a RHA data frame starts 
		RHA_sync 				: in  STD_LOGIC;
		-- RHA ADC clock
		RHA_ADC_clk				: out STD_LOGIC;

		-- Results 
		DATAOUT_DATA			: out STD_LOGIC_VECTOR(15 downto 0);
		DATAOUT_VALID			: out STD_LOGIC;
		DATAOUT_FIRST			: out STD_LOGIC;
		DATAOUT_ADDRESS			: out STD_LOGIC_VECTOR(4 downto 0);
		DATAOUT_PATTERN			: out STD_LOGIC_VECTOR(2 downto 0);
		DATAOUT_PARITY			: out STD_LOGIC;
		
		-- The usual stuff 
		Clk 					: in  STD_LOGIC;
      	Reset 					: in  STD_LOGIC
		);
end RHA_DATA_COLLECTOR;

architecture Behavioral of RHA_DATA_COLLECTOR is

	type STATE_Type is (
		-- Wait until we can lock into the RHA_sync pulse
		SM_WAIT,
		SM_WAIT_B,
		-- Positive Edge (ADC_sync and sync changes on negative edges of ADC_clck -> read on positive edges )
		-- Negative Edge (ADC_out changes on positive edges of ADC_clk -> read on negative edges)
		SM_COLLECT_A,
		SM_COLLECT_B
	);
	signal SPIMACHINE_STATE : STATE_TYPE;

	signal DATAOUT_DATA_INTERNAL 			: STD_LOGIC_VECTOR(24 downto 0) := (others => '0');
	signal DATAOUT_DATA_INTERNAL_BUFFER		: STD_LOGIC_VECTOR(24 downto 0) := (others => '0');
	signal DATAOUT_VALID_BUFFER				: STD_LOGIC := '0';
	signal DATAOUT_FIRST_BUFFER				: STD_LOGIC := '0';
	signal DATAOUT_FIRST_INTERNAL_F			: STD_LOGIC := '0';
	signal DATAOUT_VALID_INTERNAL			: STD_LOGIC := '0';
	signal RHA_SYNC_LAST1 					: STD_LOGIC := '0';
--	signal RHA_SYNC_LAST2 					: STD_LOGIC := '0';
	signal Counter 							: unsigned (4 downto 0) := (others => '0');
	signal DATAOUT_FIRST_INTERNAL 			: STD_LOGIC := '0';
	signal DATAOUT_DATA_INTERNAL_BUFFER_PRE : STD_LOGIC_VECTOR(24 downto 0) := (others => '0');
	signal RHA_ADC_clk_INTERNAL : std_logic := '0';
	signal RHA_ADC_out_BUFFER 			: STD_LOGIC := '0';
	signal RHA_sync_BUFFER 			: STD_LOGIC := '0';

begin

	-- Connect output to the internal flip flops
	-- Disassemble the data vector 
	DATAOUT_PARITY <= DATAOUT_DATA_INTERNAL_BUFFER(0); -- : out STD_LOGIC;	
	DATAOUT_DATA <= DATAOUT_DATA_INTERNAL_BUFFER(16 downto 1); -- : out STD_LOGIC_VECTOR(15 downto 0);
	DATAOUT_ADDRESS <= DATAOUT_DATA_INTERNAL_BUFFER(21 downto 17); --	: out STD_LOGIC_VECTOR(4 downto 0);
	DATAOUT_PATTERN <= DATAOUT_DATA_INTERNAL_BUFFER(24 downto 22); -- : out STD_LOGIC_VECTOR(2 downto 0);

	DATAOUT_VALID	<= DATAOUT_VALID_BUFFER;
	DATAOUT_FIRST	<= DATAOUT_FIRST_BUFFER;

	
-----------------
	RHA_ADC_clk <= RHA_ADC_clk_INTERNAL;
	
	ADC_Readout_Machine : process (Clk,Reset)
	begin
		
		if Reset = '1' then
			-- Init with empty output
			DATAOUT_DATA_INTERNAL <= (others => '0');
			DATAOUT_DATA_INTERNAL_BUFFER <= (others => '0');
			DATAOUT_DATA_INTERNAL_BUFFER_PRE <= (others => '0');
			DATAOUT_VALID_INTERNAL <= '0';
			DATAOUT_FIRST_INTERNAL_F <= '0';
			DATAOUT_VALID_BUFFER <= '0';
			DATAOUT_FIRST_BUFFER <= '0';
			DATAOUT_FIRST_INTERNAL <= '0';
			RHA_SYNC_LAST1 <= '0';
			Counter <= (others => '0');
			RHA_ADC_clk_INTERNAL <= '0';

			RHA_ADC_out_BUFFER <= '0';
			RHA_sync_BUFFER <= '0';
			SPIMACHINE_STATE <= SM_WAIT;
		elsif rising_edge(Clk) then
			-- Defaul:
			DATAOUT_DATA_INTERNAL <= DATAOUT_DATA_INTERNAL;
			DATAOUT_DATA_INTERNAL_BUFFER <= DATAOUT_DATA_INTERNAL_BUFFER_PRE;
			DATAOUT_DATA_INTERNAL_BUFFER_PRE <= DATAOUT_DATA_INTERNAL_BUFFER_PRE;

			DATAOUT_VALID_INTERNAL <= '0';
			DATAOUT_FIRST_INTERNAL_F <= '0';
			DATAOUT_VALID_BUFFER <= DATAOUT_VALID_INTERNAL;
			DATAOUT_FIRST_BUFFER <= DATAOUT_FIRST_INTERNAL_F;			

			-- Default:
			DATAOUT_FIRST_INTERNAL <= DATAOUT_FIRST_INTERNAL;
			
			RHA_SYNC_LAST1 <= RHA_sync_BUFFER;
			Counter <= Counter;

			RHA_ADC_clk_INTERNAL <= not RHA_ADC_clk_INTERNAL;

			RHA_ADC_out_BUFFER <= RHA_ADC_out;
			RHA_sync_BUFFER <= RHA_SYNC;
			
			case SPIMACHINE_STATE is

				when SM_WAIT => 

					-- Is it the first sample? 
					if (RHA_SYNC_LAST1 = '1') then
						SPIMACHINE_STATE <= SM_WAIT_B;
					else
						SPIMACHINE_STATE <= SM_WAIT;
					end if;
					DATAOUT_FIRST_INTERNAL <= '0';
					RHA_ADC_out_BUFFER <= RHA_ADC_out;
				when SM_WAIT_B => 
					SPIMACHINE_STATE <= SM_COLLECT_A;
				when SM_COLLECT_A =>	
					SPIMACHINE_STATE <= SM_COLLECT_B;
					-- Read on falling edge
					-- Bit shifting the incoming bits into result array 
					DATAOUT_DATA_INTERNAL <= DATAOUT_DATA_INTERNAL(23 downto 0) & RHA_ADC_out_BUFFER;
					
					-- Store 25bits 
					if Counter = to_unsigned(24,5) then
						DATAOUT_VALID_INTERNAL <= '1';
						DATAOUT_DATA_INTERNAL_BUFFER_PRE <= DATAOUT_DATA_INTERNAL(23 downto 0) & RHA_ADC_out_BUFFER;
						DATAOUT_DATA_INTERNAL <= (others => '0');
						
						-- Check for the first sample
						DATAOUT_FIRST_INTERNAL_F <= DATAOUT_FIRST_INTERNAL;
					else
						null;
					end if;		
					
				when SM_COLLECT_B =>	
					SPIMACHINE_STATE <= SM_COLLECT_A;
					-- Store 25bits 
					if Counter = to_unsigned(24,5) then
						Counter <= (others => '0');
						if RHA_SYNC_LAST1 = '1' then
							DATAOUT_FIRST_INTERNAL <= '1';
						else
							DATAOUT_FIRST_INTERNAL <= '0';
						end if;
					elsif (RHA_SYNC_LAST1 = '1') then
						Counter <= Counter + 1;
						DATAOUT_FIRST_INTERNAL <= '1';
					else
						Counter <= Counter + 1;
					end if;	

				when others =>
					SPIMACHINE_STATE <= SM_WAIT;
			end case;
			
		end if;
		
	end process ADC_Readout_Machine;	


end architecture Behavioral;

