----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    21:42:33 07/14/2014 
-- Design Name: 
-- Module Name:    RHA_ERROR_WATCHDOG - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity RHA_ERROR_WATCHDOG is
    Port ( 
      -- Errors (tells us if the RHA get asynchronous in their sync pulses) 
		ERROR_RHA_ASYNC_FIRST   : in  STD_LOGIC;
		ERROR_RHA_ASYNC_VALID   : in  STD_LOGIC;
		-- Is there a parity problem in this sample?
		ERROR_PARITY 				: in  STD_LOGIC;
		-- Is there a pattern problem in this sample?
		ERROR_PATTERN 				: in  STD_LOGIC;
		-- Is there an address problem in this sample?
		ERROR_ADDRESS 				: in STD_LOGIC;
		-- Was the readout too slow?
		ERROR_BUFFEROVERFLOW		: in STD_LOGIC;	 

		ERROR_RHA_ASYNC_FIRST_OUT : out  STD_LOGIC;
		ERROR_RHA_ASYNC_VALID_OUT : out  STD_LOGIC;
		-- Is there a parity problem in this sample?
		ERROR_PARITY_OUT 			: out  STD_LOGIC;
		-- Is there a pattern problem in this sample?
		ERROR_PATTERN_OUT 		: out  STD_LOGIC;
		-- Is there an address problem in this sample?
		ERROR_ADDRESS_OUT 		: out STD_LOGIC;
		-- Was the readout too slow?
		ERROR_BUFFEROVERFLOW_OUT : out STD_LOGIC;	 
		
		ERROR_ALL_OUT 		: out STD_LOGIC;
		DATAOUT_VALID_OUT			: in STD_LOGIC;
		
		Clk 			        		: in  STD_LOGIC;
        -- Reset for this module 
      Reset 		            : in  STD_LOGIC	 
	 
	 );
end RHA_ERROR_WATCHDOG;



architecture Behavioral of RHA_ERROR_WATCHDOG is
	signal ERROR_RHA_ASYNC_FIRST_OUT_INTERNAL : STD_LOGIC := '0';
	signal ERROR_RHA_ASYNC_VALID_OUT_INTERNAL : STD_LOGIC := '0';
	signal ERROR_PARITY_OUT_INTERNAL 			: STD_LOGIC := '0';
	signal ERROR_PATTERN_OUT_INTERNAL 		: STD_LOGIC := '0';
	signal ERROR_ADDRESS_OUT_INTERNAL 		: STD_LOGIC := '0';
	signal ERROR_BUFFEROVERFLOW_OUT_INTERNAL : STD_LOGIC := '0';	 
	signal ERROR_ALL_OUT_INTERNAL 		: STD_LOGIC := '0';
begin
	ERROR_RHA_ASYNC_FIRST_OUT <= ERROR_RHA_ASYNC_FIRST_OUT_INTERNAL;
	ERROR_RHA_ASYNC_VALID_OUT <= ERROR_RHA_ASYNC_VALID_OUT_INTERNAL;
	ERROR_PARITY_OUT <= ERROR_PARITY_OUT_INTERNAL;
	ERROR_PATTERN_OUT <= ERROR_PATTERN_OUT_INTERNAL;
	ERROR_ADDRESS_OUT <= ERROR_ADDRESS_OUT_INTERNAL;
	ERROR_BUFFEROVERFLOW_OUT <= ERROR_BUFFEROVERFLOW_OUT_INTERNAL;
	ERROR_ALL_OUT <= ERROR_ALL_OUT_INTERNAL;

	WATCH_ERROR_STATES : process (Clk,Reset)
	begin
		if Reset = '1' then
		
			ERROR_RHA_ASYNC_FIRST_OUT_INTERNAL <= '0';
			ERROR_RHA_ASYNC_VALID_OUT_INTERNAL <= '0';
			ERROR_PARITY_OUT_INTERNAL <= '0';
			ERROR_PATTERN_OUT_INTERNAL <= '0';
			ERROR_ADDRESS_OUT_INTERNAL <= '0';
			ERROR_BUFFEROVERFLOW_OUT_INTERNAL <= '0';
			ERROR_ALL_OUT_INTERNAL <= '0';
			
		elsif rising_edge(Clk) then
			-- DEFAULT:
			ERROR_RHA_ASYNC_FIRST_OUT_INTERNAL <= ERROR_RHA_ASYNC_FIRST_OUT_INTERNAL;
			ERROR_RHA_ASYNC_VALID_OUT_INTERNAL <= ERROR_RHA_ASYNC_VALID_OUT_INTERNAL;
			ERROR_PARITY_OUT_INTERNAL <= ERROR_PARITY_OUT_INTERNAL;
			ERROR_PATTERN_OUT_INTERNAL <= ERROR_PATTERN_OUT_INTERNAL;
			ERROR_ADDRESS_OUT_INTERNAL <= ERROR_ADDRESS_OUT_INTERNAL;
			ERROR_BUFFEROVERFLOW_OUT_INTERNAL <= ERROR_BUFFEROVERFLOW_OUT_INTERNAL;
			ERROR_ALL_OUT_INTERNAL <= ERROR_ALL_OUT_INTERNAL;

			if (ERROR_RHA_ASYNC_FIRST = '1') AND (DATAOUT_VALID_OUT = '1') then 
				ERROR_RHA_ASYNC_FIRST_OUT_INTERNAL <= '1';
				ERROR_ALL_OUT_INTERNAL <= '1';
			else
			 null;
			end if;
		 
			if (ERROR_RHA_ASYNC_VALID = '1') AND (DATAOUT_VALID_OUT = '1') then 
				ERROR_RHA_ASYNC_VALID_OUT_INTERNAL <= '1';
				ERROR_ALL_OUT_INTERNAL <= '1';
			else
			 null;
			end if;	
		
			if (ERROR_PARITY = '1') AND (DATAOUT_VALID_OUT = '1') then 
				ERROR_PARITY_OUT_INTERNAL <= '1';
				ERROR_ALL_OUT_INTERNAL <= '1';
			else
			 null;
			end if;	
		
			if (ERROR_PATTERN = '1') AND (DATAOUT_VALID_OUT = '1') then 
				ERROR_PATTERN_OUT_INTERNAL <= '1';
				ERROR_ALL_OUT_INTERNAL <= '1';
			else
			 null;
			end if;
		
			if (ERROR_ADDRESS = '1') AND (DATAOUT_VALID_OUT = '1') then 
				ERROR_ADDRESS_OUT_INTERNAL <= '1';
				ERROR_ALL_OUT_INTERNAL <= '1';
			else
			 null;
			end if;	
			
			if (ERROR_BUFFEROVERFLOW = '1') AND (DATAOUT_VALID_OUT = '1') then 
				ERROR_BUFFEROVERFLOW_OUT_INTERNAL <= '1';
				ERROR_ALL_OUT_INTERNAL <= '1';
			else
			 null;
			end if;
			
		end if;
	end process WATCH_ERROR_STATES;

end Behavioral;

