library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
use std.textio.all;

 
ENTITY IMPLANT_TOP_UUT IS
END IMPLANT_TOP_UUT;
 
ARCHITECTURE behavior OF IMPLANT_TOP_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT IMPLANT_TOP
    PORT(
	 		-- Connection to the Zarlink 
		SPI_ZL_CS 					: out STD_LOGIC;
		SPI_SDI 						: out STD_LOGIC;
		SPI_SDO 						: in  STD_LOGIC;
		SPI_CLK 						: out STD_LOGIC;		

		-- For the data stream
		RHD_MOSI 					: out STD_LOGIC_VECTOR(7 downto 0);
		RHD_MISO						: in  STD_LOGIC_VECTOR(7 downto 0);
		RHD_Clk						: out STD_LOGIC_VECTOR(7 downto 0);
		RHD_CS						: out STD_LOGIC_VECTOR(7 downto 0);
	
		-- Debugging the Zarlink 
--		DEBUG_ZARLINK_DATA		: out STD_LOGIC_VECTOR(15 downto 0);
--		DEBUG_ZARLINK_VALID		: out STD_LOGIC;

		-- Error flags 
		ERROR_ALL_OUT 				: out STD_LOGIC;
	
		-- The usual ports 
		Reset 						: in  STD_LOGIC;				
		Clk 							: in  STD_LOGIC
        );
    END COMPONENT;
    


	 		-- Connection to the Zarlink 
		signal SPI_ZL_CS 						: STD_LOGIC := '0';
		signal SPI_SDI 						: STD_LOGIC := '0';
		signal SPI_SDO 						: STD_LOGIC := '0';
		signal SPI_CLK 						: STD_LOGIC := '0';		

		signal SPI_CLK_LAST : STD_LOGIC := '0';		
		signal SPI_ZL_CS_LAST : STD_LOGIC := '0';		

		-- For the data stream
		signal RHD_MOSI 						: STD_LOGIC_VECTOR(7 downto 0)  := (others=>'0');
		signal RHD_MISO						: STD_LOGIC_VECTOR(7 downto 0)  := (others=>'0');
		signal RHD_Clk							: STD_LOGIC_VECTOR(7 downto 0)  := (others=>'0');
		signal RHD_CS							: STD_LOGIC_VECTOR(7 downto 0)  := (others=>'0');
	
		-- Debugging the Zarlink 
		signal DEBUG_ZARLINK_DATA			: STD_LOGIC_VECTOR(15 downto 0)  := (others=>'0');
		signal DEBUG_ZARLINK_VALID			: STD_LOGIC;

		-- Error flags 
		signal ERROR_ALL_OUT 				: STD_LOGIC := '0';

   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';


  	signal MasterCounter : unsigned(127 downto 0) := (others=>'0');
	signal InputCounter : unsigned(127 downto 0) := (others=>'0');	
	-- First bit is not used!!!
	signal TEST_SPI_OUTPUT : std_logic_vector (120 downto 0) := (others => '0');
	constant TEST_SPI_DUMMY : std_logic_vector (7 downto 0) := "00000000";
	constant TEST_SPI_DUMMY_II : std_logic_vector (104 downto 0) := (others => '0');
	




	-- Reverse required...  
	constant HeaderByte_1					: std_logic_vector(7 downto 0) := "00001100"; -- "00110000"; -- 48
	constant HeaderByte_2					: std_logic_vector(7 downto 0) := "10010110"; -- "01101001"; -- 105
	constant Implant_ID     				: std_logic_vector(7 downto 0) := "10101100"; -- "00110101"; -- Implant ID 53
	constant Command_RHD_COMMAND 			: std_logic_vector(7 downto 0) := "01010101"; -- "10101010";
	constant Command_Channel				: std_logic_vector(7 downto 0) := "11010101"; -- "10101011";
	constant Command_Aux						: std_logic_vector(7 downto 0) := "01010001"; -- "10001010";
	constant Command_TestMode				: std_logic_vector(7 downto 0) := "01110101"; -- "10101110"; 
	constant Command_WORKMODE				: std_logic_vector(7 downto 0) := "01011101"; -- "10111010";
	constant Command_SampleRate  			: std_logic_vector(7 downto 0) := "01010111"; -- "11101010";
	constant Command_BITMASK 				: std_logic_vector(7 downto 0) := "01010100"; -- "00101010";

BEGIN

SPI_SDO <= TEST_SPI_OUTPUT(0);
 
	-- Instantiate the Unit Under Test (UUT)
   uut: IMPLANT_TOP 
	PORT MAP (
		SPI_ZL_CS => SPI_ZL_CS,
		SPI_SDI 	=> SPI_SDI,
		SPI_SDO 	=> SPI_SDO,
		SPI_CLK 	=> SPI_CLK,

		-- For the data stream
		RHD_MOSI => RHD_MOSI,
		RHD_MISO	=> RHD_MISO,
		RHD_Clk	=> RHD_Clk,
		RHD_CS	=> RHD_CS,
	
		-- Debugging the Zarlink 
--		DEBUG_ZARLINK_DATA	=> DEBUG_ZARLINK_DATA,
--		DEBUG_ZARLINK_VALID	=> DEBUG_ZARLINK_VALID,

		-- Error flags 
		ERROR_ALL_OUT 	=> ERROR_ALL_OUT,
	
		-- The usual ports 
		Reset => Reset,
		Clk 	=> Clk
        );


 TheMasterClock : process
begin
	clockloop: for i in 0 to 1000000 loop
		Clk <= NOT Clk;
		wait for 21ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 
 
 
-- For the test the parameters in ZARLINK_CONNECT_INIT are set to:
-- constant Zarlink_WaitForReset : unsigned(23 downto 0) := to_unsigned(12,24); 
-- constant Zarlink_Retry : unsigned(16 downto 0) := to_unsigned(7,17);
-- AND in Organizer
-- constant WaitForXCyles : unsigned (17 downto 0) := to_unsigned(200,18);

TheSecondClock : process(Clk)
file result : text is out "TestData.txt";
VARIABLE  outLine  : LINE; 
begin
	if rising_edge(Clk) then

		if MasterCounter < to_unsigned(20,128) then
			Reset <= '1';
			
		else
			Reset <= '0';
		end if;

		SPI_CLK_LAST <= SPI_CLK;
		SPI_ZL_CS_LAST <= SPI_ZL_CS;
		-- Test Connection Initiation: 
		-- [->]
			-- Check if connection is exisiting (shoudn't be...)
			-- "0000000001001101" 
			-- No Connection
		if (SPI_ZL_CS_LAST = '1') AND (SPI_ZL_CS = '0') AND (InputCounter = to_unsigned(0,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- we will se for a Zarlink connection init:
			-- WAIT
			-- "0100101011111010" ZC_INITIATE_RESET
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(1,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- WAIT 
			-- "0000011011010100" Set SPI Speed
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(2,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "1111000001011000" Disable Watchdog
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(3,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0000011011010100" Set SPI Speed
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(4,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0000000001001101" Check IRQ_RADIOREADY
		-- Let the first IRQ RADIOREADY fail and then answer correctly 
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(5,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(6,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00010000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "1111000011101000" ZC_REG_MAC_MODUSER
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(7,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;			
			-- "1010000010011000" ZC_REG_MAC_CHANNEL
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(8,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0111000000100000" ZC_REG_REG_TXBUFFER_BSIZE_A
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(9,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0111100001000000" ZC_REG_REG_RXBUFFER_BSIZE_A
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(10,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "1111100010100000" ZC_REG_TXBUFF_MAXPACKSIZE_A
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(11,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0100000001000100" ZC_ALLOW_HOUSEKEEPINGMESSAGES
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(12,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0101010101000010" ZC_IMD_COPY_ID_A
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(13,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0101010111000010" ZC_IMD_COPY_ID_B
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(14,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0101010100100010" ZC_IMD_COPY_ID_C
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(15,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0101010111001000" ZC_IMD_ID_A
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(16,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0101010100101000" ZC_IMD_ID_B
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(17,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0101010110101000" ZC_IMD_ID_C
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(18,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0000000001010100" ZC_REG_MAC_INITCOM_A_IMD
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(19,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- "0000000001001101" ZC_WAIT_IRQ_LINKREADY_A_REQ
			-- Let the first IRQ LINKREADY fail and then answer correctly
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(20,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(21,128)) then
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00100000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			-- Yes Connection
			-- [2] = '1'
			-- Everything is still fine... (LINK READY)
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(22,128)) then			
			-- Yes Connection
			-- [2] = '1'
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00100000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
		-- Test Connection Initiation 
		-- [<-] CHECKED AND OK!!! (DON't touch ZarlinkConnectionInit and CheckConnection anymore)

-- RHD_COMMAND
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(23,128)) then	
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;			
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(24,128)) then	
			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "11100011" & "01010101" 
									 & "10001100" & Command_RHD_COMMAND & Implant_ID 
									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;

		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(25,128)) then	
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;			
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(26,128)) then	
			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "00000000" & "10000000" 
									 & "00000000" & Command_Channel & Implant_ID 
									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;				
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(27,128)) then	
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;			
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(28,128)) then	
			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "11100011" & "01010101" 
									 & "10001100" & Command_RHD_COMMAND & Implant_ID 
									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
-- Command_WORKMODE ON (On off on tested !)
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(29,128)) then	
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;			
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(30,128)) then	
			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "00000000" & "00000000" 
									 & "10000000" & Command_WORKMODE & Implant_ID 
									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(31,128)) then	
			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;			
		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(32,128)) then	
			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "00000000" & "00000000" 
									 & "00000000" & "11100011" & "01010101" 
									 & "10001100" & Command_RHD_COMMAND & Implant_ID 
									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
			InputCounter <= InputCounter + 1;
			
--		-- Now the system checks the ingoing buffer 
--		-- if there is nothing it checks the Linkready
--		-- and if this fails then we start all over again ... 
--		-- Thus we have one thing in the incoming buffer 
---- RHD 0		
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(23,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(24,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "01111111" & "00000000" 
--									 & "00000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(25,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(26,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10111111" & "10000000" 
--									 & "00000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(27,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(28,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11011111" & "01000000" 
--									 & "00000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(29,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(30,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11101111" & "11000000" 
--									 & "00000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
-------------------			
---- RHD 1		
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(31,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(32,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00111111" & "00000000" 
--									 & "10000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(33,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(34,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00111111" & "10000000" 
--									 & "10000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(35,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(36,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "01011111" & "01000000" 
--									 & "10000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(37,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(38,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "01101111" & "11000000" 
--									 & "10000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
-------------------			
---- RHD 2		
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(39,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(40,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00111111" & "00000000" 
--									 & "01000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(41,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(42,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10111111" & "10000000" 
--									 & "01000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(43,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(44,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10011111" & "01000000" 
--									 & "01000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(45,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(46,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10101111" & "11000000" 
--									 & "01000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
-------------------			
---- RHD 3		
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(47,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(48,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "01011111" & "00000000" 
--									 & "11000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(49,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(50,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10011111" & "10000000" 
--									 & "11000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(51,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(52,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11011111" & "01000000" 
--									 & "11000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
--			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(53,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(54,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11101111" & "11000000" 
--									 & "11000000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
-------------------			
---- RHD 4
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(55,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(56,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "01111110" & "00000000" 
--									 & "00100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(57,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(58,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10111110" & "10000000" 
--									 & "00100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(59,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(60,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11011110" & "01000000" 
--									 & "00100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(61,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(62,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11101110" & "11000000" 
--									 & "00100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
-------------------			
---- RHD 5		
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(63,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(64,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00111101" & "00000000" 
--									 & "10100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(65,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(66,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00111101" & "10000000" 
--									 & "10100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(67,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(68,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "01011101" & "01000000" 
--									 & "10100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(69,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(70,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "01101101" & "11000000" 
--									 & "10100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
-------------------			
---- RHD 6		
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(71,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(72,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00111011" & "00000000" 
--									 & "01100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(73,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(74,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10111011" & "10000000" 
--									 & "01100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(75,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(76,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10011011" & "01000000" 
--									 & "01100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(77,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(78,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10101011" & "11000000" 
--									 & "01100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
-------------------			
---- RHD 7		
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(79,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(80,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "01010111" & "00000000" 
--									 & "11100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(81,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(82,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "10010111" & "10000000" 
--									 & "11100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;									 
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(83,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(84,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11010111" & "01000000" 
--									 & "11100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
--			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(85,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(86,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11100111" & "11000000" 
--									 & "11100000" & Command_Channel & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
-------------------			
---- AUX !!!
---- AUX 0, 1 and 2 (Warning this gives not the desired number of response packages but it works!!!)
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(87,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(88,128)) then	
--			TEST_SPI_OUTPUT <= "0" & HeaderByte_1 & "00000000" & "10100000" 
--									 & "10000000" & Command_Aux & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & "11100000" 
--									 & "00000000" & Command_Aux & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;		
---- AUX 2 (rest)
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(89,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(90,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "01100000" & "01000000" & Command_Aux 
--									 & Implant_ID & HeaderByte_2 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;
---- AUX 3
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(91,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(92,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "11000000" 
--									 & "11000000" & Command_Aux & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;	
---- AUX 4
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(93,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(94,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "11100000" 
--									 & "00100000" & Command_Aux & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;	
---- AUX 5
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(95,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(96,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "10100000" 
--									 & "10100000" & Command_Aux & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;	
---- AUX 6
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(97,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(98,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "01100000" 
--									 & "01100000" & Command_Aux & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;	
---- AUX 7
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(99,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(100,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "11100000" 
--									 & "11100000" & Command_Aux & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;	
---- SAMPLE RATE
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(101,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(102,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "11000000" & "00000000" & "00000000" 
--									 & "00000000" & Command_SampleRate & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;
---- BITMASK
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(103,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(104,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "11100011" 
--									 & "11100000" & Command_BITMASK & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;	
---- Test Mode Off
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(105,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(106,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & Command_TestMode & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;	
---- Test Mode On
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(107,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(108,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "10000000" & Command_TestMode & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;	
--
---- RHD_COMMAND
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(109,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(110,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11100011" & "01010101" 
--									 & "10001100" & Command_RHD_COMMAND & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;
--
---- Command_WORKMODE ON (On off on tested !)
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(111,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(112,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "10000000" & Command_WORKMODE & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(113,128)) then	
--			TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
--		elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(114,128)) then	
--			TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "00000000" & "00000000" 
--									 & "00000000" & "11101011" & "01011101" 
--									 & "10001100" & Command_RHD_COMMAND & Implant_ID 
--									 & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
--			InputCounter <= InputCounter + 1;			
		else
			if (SPI_CLK = '0') AND (SPI_CLK_LAST = '1') then
				TEST_SPI_OUTPUT <= "0" & TEST_SPI_OUTPUT(120 downto 1);
			else
				TEST_SPI_OUTPUT <= TEST_SPI_OUTPUT;
			end if;
		end if;

		if (SPI_CLK = '1') AND (SPI_CLK_LAST = '0') then
			if SPI_SDI = '1' then
				write(outLine,string'("1"));
			elsif SPI_SDI = '0' then
				write(outLine,string'("0"));
			else 
				null;
			end if;
		end if;
		
		if (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') then
			writeline(result,outLine);
		end if;		
		
		MasterCounter <= MasterCounter + 1;
	end if;
end process TheSecondClock;	


END;
