library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

ENTITY RHA_ARRAY_UUT IS
END RHA_ARRAY_UUT;
 
ARCHITECTURE behavior OF RHA_ARRAY_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)


   --Inputs
   signal RHA_ADC_out : std_logic_vector(7 downto 0) := (others => '0');
   signal RHA_sync : std_logic_vector(7 downto 0) := (others => '0');
   signal RHA_CHANNEL_MASK : std_logic_vector(127 downto 0) := (others => '0');
   signal Clk : std_logic := '0';
   signal Reset_RHA : std_logic := '0';
   signal Reset : std_logic := '0';
	signal ReadRequest : std_logic := '1';

 	--Outputs
   signal RHA_ADC_clk : std_logic_vector(7 downto 0);
	signal DATAOUT_DATA_OUT : std_logic_vector(15 downto 0);
   signal DATAOUT_VALID_OUT : std_logic;
   signal Channel_Number_OUT : std_logic_vector(3 downto 0);

  	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 

	signal Clk_Fast : STD_LOGIC := '0';
	signal ExportNext128ChannelBlock : STD_LOGIC := '1';
	signal BITMASK_SAMPLE : std_logic_vector(15 downto 0) := "0000111111111111";
	signal DeviceID : std_logic_vector(7 downto 0) := "11001101";
	signal FIFO_OUT_DATA : STD_LOGIC_VECTOR(7 downto 0);
	signal FIFO_OUT_VALID : STD_LOGIC;
	signal ADCResolution	: std_logic_vector(3 downto 0) := "1101";
	signal SampleRate : std_logic_vector(7 downto 0) := "11000111";
	signal Filter : std_logic_vector(3 downto 0) := "1101";

	signal NewBlock : std_logic := '0';

BEGIN
	RHA_CHANNEL_MASK <= 	
			"1111111111111111" &
			"1111111111111111" &
			"0000000000000101" &
			"1111111111111111" &
			"1111111111111111" &
			"1111111111111111" &
			"0000000000000110" &			
			"1111111111111111" ;
			
	-- Instantiate the Unit Under Test (UUT)
   uut: entity work.RHA_ARRAY 
	PORT MAP (
          RHA_ADC_out => RHA_ADC_out,
          RHA_sync => RHA_sync,
          RHA_ADC_clk => RHA_ADC_clk,
          RHA_CHANNEL_MASK => RHA_CHANNEL_MASK,
          DATAOUT_DATA_OUT => DATAOUT_DATA_OUT,
          DATAOUT_VALID_OUT => DATAOUT_VALID_OUT,
          NewBlock						=> NewBlock,
			 ReadRequest => ReadRequest,
			 Clk_SLOW		        		=> Clk,
          Clk => Clk_Fast,
          Reset_RHA => Reset_RHA,
          Reset => Reset
        );

	RHA_TO_ZL_CONVERTER: entity work.RHA_TO_ZL_CONVERTER
    Port MAP (

	   DATAOUT_DATA_OUT			=> DATAOUT_DATA_OUT,
		DATAOUT_VALID_OUT			=> DATAOUT_VALID_OUT,
		NewBlock						=> NewBlock,
		ReadRequest 				=> ReadRequest,
		FIFO_OUT_DATA 				=> FIFO_OUT_DATA,
		FIFO_OUT_VALID   			=> FIFO_OUT_VALID,
		ChannelMask     			=> RHA_CHANNEL_MASK,
		ADCResolution   			=> ADCResolution,
		SampleRate      			=> SampleRate,
		Filter          			=> Filter,
		DeviceID 					=> DeviceID,       					
		BITMASK_SAMPLE				=> BITMASK_SAMPLE, 	
		ExportNext128ChannelBlock => ExportNext128ChannelBlock, 
		Clk 			        		=> Clk_Fast,
		Clk_SLOW		        		=> Clk,
      Reset 		            => Reset	
	  );

   DataGen_0: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 0
			)
	PORT MAP (
          ADC_sync => open,
          sync => RHA_sync(0),
          ADC_out => RHA_ADC_out(0),
          Reset => Reset_RHA,
          Clk => RHA_ADC_clk(0)
        );  

   DataGen_1: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 1
			)
	PORT MAP (
          ADC_sync => open,
          sync => RHA_sync(1),
          ADC_out => RHA_ADC_out(1),
          Reset => Reset_RHA,
          Clk => RHA_ADC_clk(1)
        ); 

   DataGen_2: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 2
			)
	PORT MAP (
          ADC_sync => open,
          sync => RHA_sync(2),
          ADC_out => RHA_ADC_out(2),
          Reset => Reset_RHA,
          Clk => RHA_ADC_clk(2)
        ); 

   DataGen_3: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 3
			)
	PORT MAP (
          ADC_sync => open,
          sync => RHA_sync(3),
          ADC_out => RHA_ADC_out(3),
          Reset => Reset_RHA,
          Clk => RHA_ADC_clk(3)
        ); 
		  
   DataGen_4: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 4
			)
	PORT MAP (
          ADC_sync => open,
          sync => RHA_sync(4),
          ADC_out => RHA_ADC_out(4),
          Reset => Reset_RHA,
          Clk => RHA_ADC_clk(4)
        ); 

   DataGen_5: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 5
			)
	PORT MAP (
          ADC_sync => open,
          sync => RHA_sync(5),
          ADC_out => RHA_ADC_out(5),
          Reset => Reset_RHA,
          Clk => RHA_ADC_clk(5)
        ); 

   DataGen_6: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 6
			)
	PORT MAP (
          ADC_sync => open,
          sync => RHA_sync(6),
          ADC_out => RHA_ADC_out(6),
          Reset => Reset_RHA,
          Clk => RHA_ADC_clk(6)
        ); 

   DataGen_7: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 7
			)
	PORT MAP (
          ADC_sync => open,
          sync => RHA_sync(7),
          ADC_out => RHA_ADC_out(7),
          Reset => Reset_RHA,
          Clk => RHA_ADC_clk(7)
        ); 

TheMasterClock : process
begin
	clockloop: for i in 0 to 200000 loop
		Clk_Fast <= NOT Clk_Fast;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

TheMasterClock_Slow : process
begin
	clockloop: for i in 0 to 50000 loop
		Clk <= NOT Clk;
		wait for 16ns;
	end loop clockloop;
	wait;
end process TheMasterClock_Slow; 



TheSecondClock : process(Clk_Fast) 
begin
	if rising_edge(Clk_Fast) then

		if MasterCounter < to_unsigned(20,128) then
			Reset <= '1';
			Reset_RHA <= '1';
		else
			Reset <= '0';
			Reset_RHA <= '0';
		end if;

		
--		if MasterCounter = to_unsigned(12,128) then
--			RHA_sync <= '1';
--		elsif MasterCounter = to_unsigned(13,128) then
--			RHA_sync <= '1';
--		elsif MasterCounter = to_unsigned(812,128) then
--			RHA_sync <= '1';
--		elsif MasterCounter = to_unsigned(813,128) then
--			RHA_sync <= '1';			
--		else
--			RHA_sync <= '0';
--		end if;
		
--		if (DATAOUT_VALID_LAST = '0') AND (DATAOUT_VALID = '1') then
--			RHA_ADC_out <= NOT RHA_ADC_out;
--		else
--			RHA_ADC_out <= RHA_ADC_out;
--		end if;
		
		MasterCounter <= MasterCounter + 1;
	end if;
end process TheSecondClock;	
 

END;
