library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

ENTITY RHA_DATA_COLLECTOR_UUT IS
END RHA_DATA_COLLECTOR_UUT;
 
ARCHITECTURE behavior OF RHA_DATA_COLLECTOR_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT RHA_DATA_COLLECTOR
    PORT(
         RHA_ADC_out : IN  std_logic;
         RHA_sync : IN  std_logic;
         RHA_ADC_clk : OUT  std_logic;
         DATAOUT_DATA : OUT  std_logic_vector(15 downto 0);
         DATAOUT_ADDRESS : OUT  std_logic_vector(4 downto 0);
         DATAOUT_PATTERN : OUT  std_logic_vector(2 downto 0);
         DATAOUT_PARITY : OUT  std_logic;
         DATAOUT_VALID : OUT  std_logic;
         DATAOUT_FIRST : OUT  std_logic;
         Clk : IN  std_logic;
         Reset : IN  std_logic
        );
    END COMPONENT;


   --Inputs
   signal RHA_ADC_out : std_logic := '0';
   signal RHA_sync : std_logic := '0';
   signal Clk : std_logic := '0';
   signal Reset : std_logic := '0';

 	--Outputs
   signal RHA_ADC_clk : std_logic;
   signal DATAOUT_DATA : std_logic_vector(15 downto 0);
   signal DATAOUT_ADDRESS : std_logic_vector(4 downto 0);
   signal DATAOUT_PATTERN : std_logic_vector(2 downto 0);
   signal DATAOUT_PARITY : std_logic;
   signal DATAOUT_VALID : std_logic;
   signal DATAOUT_FIRST : std_logic;

   signal DATAOUT_VALID_LAST : std_logic := '0';
   signal DATAOUT_FIRST_LAST : std_logic := '0';

  	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
	
	

	signal ADC_sync : std_logic := '0';
	

	signal Channel_Number		: unsigned(3 downto 0) := (others=>'0'); 

	signal ERROR_PARITY			: STD_LOGIC := '0';
	signal ERROR_PATTERN			: STD_LOGIC := '0';
	signal ERROR_ADDRESS			: STD_LOGIC := '0';
	
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: RHA_DATA_COLLECTOR PORT MAP (
          RHA_ADC_out => RHA_ADC_out,
          RHA_sync => RHA_sync,
          RHA_ADC_clk => RHA_ADC_clk,
          DATAOUT_DATA => DATAOUT_DATA,
          DATAOUT_ADDRESS => DATAOUT_ADDRESS,
          DATAOUT_PATTERN => DATAOUT_PATTERN,
          DATAOUT_PARITY => DATAOUT_PARITY,
          DATAOUT_VALID => DATAOUT_VALID,
          DATAOUT_FIRST => DATAOUT_FIRST,
          Clk => Clk,
          Reset => Reset
        );

	-- Instantiate the Unit Under Test (UUT)
   DataGen: entity work.rha_simulator 
	Generic MAP (
			IDOffset => 0
			)
	PORT MAP (
          ADC_sync => ADC_sync,
          sync => RHA_sync,
          ADC_out => RHA_ADC_out,
          Reset => Reset,
          Clk => RHA_ADC_clk
        );    

	DataChecker: entity work. RHA_DATA_CHECKER 
    Port MAP (
		DATAOUT_DATA => DATAOUT_DATA,
		DATAOUT_ADDRESS => DATAOUT_ADDRESS,
		DATAOUT_PATTERN => DATAOUT_PATTERN,
		DATAOUT_PARITY	=> DATAOUT_PARITY,

		Channel_Number	=> std_logic_vector(Channel_Number),

		ERROR_PARITY => ERROR_PARITY,
		ERROR_PATTERN => ERROR_PATTERN,
		ERROR_ADDRESS => ERROR_ADDRESS
	 );


TheMasterClock : process
begin
	clockloop: for i in 0 to 50000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

TheSecondClock : process(clk) 
begin
	if rising_edge(clk) then

		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;

		DATAOUT_VALID_LAST <= DATAOUT_VALID;
		DATAOUT_FIRST_LAST <= DATAOUT_FIRST;

		
		if (DATAOUT_FIRST = '1') AND (DATAOUT_FIRST_LAST = '0') then
			Channel_Number <= (others => '0');
		elsif (DATAOUT_VALID = '1') AND (DATAOUT_VALID_LAST = '0') then
			Channel_Number <= Channel_Number + 1;
		else
			Channel_Number <= Channel_Number;
		end if;
		
--		if MasterCounter = to_unsigned(12,128) then
--			RHA_sync <= '1';
--		elsif MasterCounter = to_unsigned(13,128) then
--			RHA_sync <= '1';
--		elsif MasterCounter = to_unsigned(812,128) then
--			RHA_sync <= '1';
--		elsif MasterCounter = to_unsigned(813,128) then
--			RHA_sync <= '1';			
--		else
--			RHA_sync <= '0';
--		end if;
		
--		if (DATAOUT_VALID_LAST = '0') AND (DATAOUT_VALID = '1') then
--			RHA_ADC_out <= NOT RHA_ADC_out;
--		else
--			RHA_ADC_out <= RHA_ADC_out;
--		end if;
		
		MasterCounter <= MasterCounter + 1;
	end if;
end process TheSecondClock;	

END;
