library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

ENTITY RHA_TESTMODULE_UUT IS
END RHA_TESTMODULE_UUT;
 
ARCHITECTURE behavior OF RHA_TESTMODULE_UUT IS 
 

	signal RHD_MOSI_OUTSIDE			: STD_LOGIC_VECTOR (7 downto 0) := (others=>'0');
	signal RHD_MISO_OUTSIDE			: STD_LOGIC_VECTOR (7 downto 0) := (others=>'0');
	signal RHD_Clk_OUTSIDE			: STD_LOGIC_VECTOR (7 downto 0) := (others=>'0');
	signal RHD_CS_OUTSIDE			: STD_LOGIC_VECTOR (7 downto 0) := (others=>'0');

   signal MasterCounter : unsigned(127 downto 0) := (others=>'0');
		
   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';


	signal NewBlock_IN : std_logic := '0';
	signal NewBlock_OUT : std_logic := '0';
	signal NewBlock_IN_LAST : std_logic := '0';
	signal Command_IN : std_logic := '0';
	signal Command_OUT : std_logic := '0';

	signal END_PAUSE	: std_logic := '0';


		-- Results we get back 
	signal DATAOUT_DATA			: STD_LOGIC_VECTOR(15 downto 0) := (others=>'0');
	signal DATAOUT_VALID			: STD_LOGIC  := '0';

		-- The ports for sending the data out
	signal DATAIN_DATA			: STD_LOGIC_VECTOR(15 downto 0) := (others=>'0');
	signal DATAIN_VALID			: STD_LOGIC  := '1';
	signal DATAIN_USED			: STD_LOGIC  := '0';

	signal Counter : unsigned (7 downto 0) := (others => '0');
	signal InputCounter_LAST : unsigned (5 downto 0) := (others => '0');
	signal InputCounter : unsigned(5 downto 0) :=  (others => '0');
	
BEGIN

	-- Instantiate the Unit Under Test (UUT)
   uut: entity work.RHD_TESTMODULE 
		PORT MAP (
			RHD_MOSI_OUTSIDE	=> RHD_MOSI_OUTSIDE,
			RHD_MISO_OUTSIDE => RHD_MISO_OUTSIDE,
			RHD_Clk_OUTSIDE => RHD_Clk_OUTSIDE,
			RHD_CS_OUTSIDE => RHD_CS_OUTSIDE,
			Reset => Reset
        );


		MyRHD_DATA_COLLECTOR : entity work.RHD_DATA_COLLECTOR
    Port MAP( 
		RHD_MOSI_OUTSIDE => RHD_MOSI_OUTSIDE(0),
		RHD_MISO_OUTSIDE => RHD_MISO_OUTSIDE(0),
		RHD_Clk_OUTSIDE => RHD_Clk_OUTSIDE(0),
		RHD_CS_OUTSIDE => RHD_CS_OUTSIDE(0),

		-- Results we get back 
		DATAOUT_DATA => DATAOUT_DATA,
		DATAOUT_VALID => DATAOUT_VALID,

		-- The ports for sending the data out
		DATAIN_DATA => DATAIN_DATA,
		DATAIN_VALID => DATAIN_VALID,
		DATAIN_USED	=> DATAIN_USED,

		NewBlock_IN				=> NewBlock_IN,
		NewBlock_OUT 			=> NewBlock_OUT,
		
		Command_IN				=> Command_IN,
		Command_OUT				=> Command_OUT,

		END_PAUSE				=> END_PAUSE,
		
		-- The usual stuff 
		-- Needs to be double the RHA ADC clock 
		Clk => Clk,
      Reset => Reset
		);



TheMasterClock : process
begin
	clockloop: for i in 0 to 10000 loop
		Clk <= NOT Clk;
		wait for 21ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

DATAIN_DATA <= "1010101010" & std_logic_vector(InputCounter);

TheSecondClock : process(Clk)
begin
	if rising_edge(Clk) then

		
		
		if MasterCounter < to_unsigned(20,128) then
			Reset <= '1';
			
		else
			Reset <= '0';
		end if;
		
		if Counter = to_unsigned(40,8) then
			Counter <= (others => '0');
			END_PAUSE <= '1';
		else
			Counter <= Counter + 1;
			END_PAUSE <= '0';
		end if;
		
		if DATAIN_USED = '1' then
			InputCounter <= InputCounter + 1;
			InputCounter_LAST <= InputCounter;
			if NewBlock_IN = '1' then
				Command_IN <= '1';
			else
				Command_IN <= '0';		
			end if;			
		else
			InputCounter <= InputCounter;
			InputCounter_LAST <= InputCounter_LAST;
			Command_IN <= Command_IN;
		end if ;

		if (InputCounter(2) = '1') AND (InputCounter_LAST(2) = '0') then
			NewBlock_IN <= '1';
		else
			NewBlock_IN <= '0';
		end if;
		


		
		MasterCounter <= MasterCounter + 1;
	end if;
end process TheSecondClock;	

END;
