-- Version: v11.3 11.3.0.112

library ieee;
use ieee.std_logic_1164.all;
library igloo;
use igloo.all;

entity FIFO is

    port( DIN        : in    std_logic_vector(7 downto 0);
          DOUT       : out   std_logic_vector(7 downto 0);
          WR_EN      : in    std_logic;
          RD_EN      : in    std_logic;
          WR_CLK     : in    std_logic;
          RD_CLK     : in    std_logic;
          FULL       : out   std_logic;
          EMPTY      : out   std_logic;
          RST        : in    std_logic;
          PROG_EMPTY : out   std_logic;
          PROG_FULL  : out   std_logic
        );

end FIFO;

architecture DEF_ARCH of FIFO is 

  component OR2
    port( A : in    std_logic := 'U';
          B : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component FIFO4K18
    port( AEVAL11 : in    std_logic := 'U';
          AEVAL10 : in    std_logic := 'U';
          AEVAL9  : in    std_logic := 'U';
          AEVAL8  : in    std_logic := 'U';
          AEVAL7  : in    std_logic := 'U';
          AEVAL6  : in    std_logic := 'U';
          AEVAL5  : in    std_logic := 'U';
          AEVAL4  : in    std_logic := 'U';
          AEVAL3  : in    std_logic := 'U';
          AEVAL2  : in    std_logic := 'U';
          AEVAL1  : in    std_logic := 'U';
          AEVAL0  : in    std_logic := 'U';
          AFVAL11 : in    std_logic := 'U';
          AFVAL10 : in    std_logic := 'U';
          AFVAL9  : in    std_logic := 'U';
          AFVAL8  : in    std_logic := 'U';
          AFVAL7  : in    std_logic := 'U';
          AFVAL6  : in    std_logic := 'U';
          AFVAL5  : in    std_logic := 'U';
          AFVAL4  : in    std_logic := 'U';
          AFVAL3  : in    std_logic := 'U';
          AFVAL2  : in    std_logic := 'U';
          AFVAL1  : in    std_logic := 'U';
          AFVAL0  : in    std_logic := 'U';
          WD17    : in    std_logic := 'U';
          WD16    : in    std_logic := 'U';
          WD15    : in    std_logic := 'U';
          WD14    : in    std_logic := 'U';
          WD13    : in    std_logic := 'U';
          WD12    : in    std_logic := 'U';
          WD11    : in    std_logic := 'U';
          WD10    : in    std_logic := 'U';
          WD9     : in    std_logic := 'U';
          WD8     : in    std_logic := 'U';
          WD7     : in    std_logic := 'U';
          WD6     : in    std_logic := 'U';
          WD5     : in    std_logic := 'U';
          WD4     : in    std_logic := 'U';
          WD3     : in    std_logic := 'U';
          WD2     : in    std_logic := 'U';
          WD1     : in    std_logic := 'U';
          WD0     : in    std_logic := 'U';
          WW0     : in    std_logic := 'U';
          WW1     : in    std_logic := 'U';
          WW2     : in    std_logic := 'U';
          RW0     : in    std_logic := 'U';
          RW1     : in    std_logic := 'U';
          RW2     : in    std_logic := 'U';
          RPIPE   : in    std_logic := 'U';
          WEN     : in    std_logic := 'U';
          REN     : in    std_logic := 'U';
          WBLK    : in    std_logic := 'U';
          RBLK    : in    std_logic := 'U';
          WCLK    : in    std_logic := 'U';
          RCLK    : in    std_logic := 'U';
          RESET   : in    std_logic := 'U';
          ESTOP   : in    std_logic := 'U';
          FSTOP   : in    std_logic := 'U';
          RD17    : out   std_logic;
          RD16    : out   std_logic;
          RD15    : out   std_logic;
          RD14    : out   std_logic;
          RD13    : out   std_logic;
          RD12    : out   std_logic;
          RD11    : out   std_logic;
          RD10    : out   std_logic;
          RD9     : out   std_logic;
          RD8     : out   std_logic;
          RD7     : out   std_logic;
          RD6     : out   std_logic;
          RD5     : out   std_logic;
          RD4     : out   std_logic;
          RD3     : out   std_logic;
          RD2     : out   std_logic;
          RD1     : out   std_logic;
          RD0     : out   std_logic;
          FULL    : out   std_logic;
          AFULL   : out   std_logic;
          EMPTY   : out   std_logic;
          AEMPTY  : out   std_logic
        );
  end component;

  component NAND2A
    port( A : in    std_logic := 'U';
          B : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component NAND2
    port( A : in    std_logic := 'U';
          B : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component INV
    port( A : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component AND2
    port( A : in    std_logic := 'U';
          B : in    std_logic := 'U';
          Y : out   std_logic
        );
  end component;

  component GND
    port(Y : out std_logic); 
  end component;

  component VCC
    port(Y : out std_logic); 
  end component;

    signal \FULL\, \EMPTY\, WEBP, RESETP, WRITE_FSTOP_ENABLE, 
        WRITE_ENABLE_I, READ_ESTOP_ENABLE, READ_ENABLE_I, 
        \FULLX_I[0]\, \EMPTYX_I[0]\, \AFULLX_I[0]\, 
        \AEMPTYX_I[0]\, \FULLX_I[1]\, \EMPTYX_I[1]\, 
        \AFULLX_I[1]\, \AEMPTYX_I[1]\, \FULLX_I[2]\, 
        \EMPTYX_I[2]\, \AFULLX_I[2]\, \AEMPTYX_I[2]\, 
        \FULLX_I[3]\, \EMPTYX_I[3]\, \AFULLX_I[3]\, 
        \AEMPTYX_I[3]\, \FULLX_I[4]\, \EMPTYX_I[4]\, 
        \AFULLX_I[4]\, \AEMPTYX_I[4]\, \FULLX_I[5]\, 
        \EMPTYX_I[5]\, \AFULLX_I[5]\, \AEMPTYX_I[5]\, 
        \FULLX_I[6]\, \EMPTYX_I[6]\, \AFULLX_I[6]\, 
        \AEMPTYX_I[6]\, \FULLX_I[7]\, \EMPTYX_I[7]\, 
        \AFULLX_I[7]\, \AEMPTYX_I[7]\, OR2_23_Y, OR2_19_Y, 
        OR2_12_Y, OR2_16_Y, OR2_18_Y, OR2_11_Y, OR2_15_Y, OR2_4_Y, 
        OR2_9_Y, OR2_3_Y, OR2_6_Y, OR2_21_Y, OR2_20_Y, OR2_5_Y, 
        OR2_22_Y, OR2_8_Y, OR2_10_Y, OR2_7_Y, OR2_14_Y, OR2_0_Y, 
        OR2_1_Y, OR2_13_Y, OR2_17_Y, OR2_2_Y, \VCC\, \GND\
         : std_logic;
    signal GND_power_net1 : std_logic;
    signal VCC_power_net1 : std_logic;

begin 

    FULL <= \FULL\;
    EMPTY <= \EMPTY\;
    \GND\ <= GND_power_net1;
    \VCC\ <= VCC_power_net1;

    OR2_EMPTY : OR2
      port map(A => OR2_23_Y, B => OR2_19_Y, Y => \EMPTY\);
    
    \FIFOBLOCK[4]\ : FIFO4K18
      port map(AEVAL11 => \GND\, AEVAL10 => \GND\, AEVAL9 => 
        \GND\, AEVAL8 => \GND\, AEVAL7 => \GND\, AEVAL6 => \GND\, 
        AEVAL5 => \GND\, AEVAL4 => \GND\, AEVAL3 => \VCC\, AEVAL2
         => \VCC\, AEVAL1 => \GND\, AEVAL0 => \VCC\, AFVAL11 => 
        \VCC\, AFVAL10 => \VCC\, AFVAL9 => \VCC\, AFVAL8 => \GND\, 
        AFVAL7 => \VCC\, AFVAL6 => \VCC\, AFVAL5 => \GND\, AFVAL4
         => \VCC\, AFVAL3 => \VCC\, AFVAL2 => \GND\, AFVAL1 => 
        \GND\, AFVAL0 => \GND\, WD17 => \GND\, WD16 => \GND\, 
        WD15 => \GND\, WD14 => \GND\, WD13 => \GND\, WD12 => 
        \GND\, WD11 => \GND\, WD10 => \GND\, WD9 => \GND\, WD8
         => \GND\, WD7 => \GND\, WD6 => \GND\, WD5 => \GND\, WD4
         => \GND\, WD3 => \GND\, WD2 => \GND\, WD1 => \GND\, WD0
         => DIN(4), WW0 => \GND\, WW1 => \GND\, WW2 => \GND\, RW0
         => \GND\, RW1 => \GND\, RW2 => \GND\, RPIPE => \GND\, 
        WEN => WRITE_ENABLE_I, REN => READ_ENABLE_I, WBLK => 
        \GND\, RBLK => \GND\, WCLK => WR_CLK, RCLK => RD_CLK, 
        RESET => RESETP, ESTOP => \VCC\, FSTOP => \VCC\, RD17 => 
        OPEN, RD16 => OPEN, RD15 => OPEN, RD14 => OPEN, RD13 => 
        OPEN, RD12 => OPEN, RD11 => OPEN, RD10 => OPEN, RD9 => 
        OPEN, RD8 => OPEN, RD7 => OPEN, RD6 => OPEN, RD5 => OPEN, 
        RD4 => OPEN, RD3 => OPEN, RD2 => OPEN, RD1 => OPEN, RD0
         => DOUT(4), FULL => \FULLX_I[4]\, AFULL => \AFULLX_I[4]\, 
        EMPTY => \EMPTYX_I[4]\, AEMPTY => \AEMPTYX_I[4]\);
    
    \FIFOBLOCK[5]\ : FIFO4K18
      port map(AEVAL11 => \GND\, AEVAL10 => \GND\, AEVAL9 => 
        \GND\, AEVAL8 => \GND\, AEVAL7 => \GND\, AEVAL6 => \GND\, 
        AEVAL5 => \GND\, AEVAL4 => \GND\, AEVAL3 => \VCC\, AEVAL2
         => \VCC\, AEVAL1 => \GND\, AEVAL0 => \VCC\, AFVAL11 => 
        \VCC\, AFVAL10 => \VCC\, AFVAL9 => \VCC\, AFVAL8 => \GND\, 
        AFVAL7 => \VCC\, AFVAL6 => \VCC\, AFVAL5 => \GND\, AFVAL4
         => \VCC\, AFVAL3 => \VCC\, AFVAL2 => \GND\, AFVAL1 => 
        \GND\, AFVAL0 => \GND\, WD17 => \GND\, WD16 => \GND\, 
        WD15 => \GND\, WD14 => \GND\, WD13 => \GND\, WD12 => 
        \GND\, WD11 => \GND\, WD10 => \GND\, WD9 => \GND\, WD8
         => \GND\, WD7 => \GND\, WD6 => \GND\, WD5 => \GND\, WD4
         => \GND\, WD3 => \GND\, WD2 => \GND\, WD1 => \GND\, WD0
         => DIN(5), WW0 => \GND\, WW1 => \GND\, WW2 => \GND\, RW0
         => \GND\, RW1 => \GND\, RW2 => \GND\, RPIPE => \GND\, 
        WEN => WRITE_ENABLE_I, REN => READ_ENABLE_I, WBLK => 
        \GND\, RBLK => \GND\, WCLK => WR_CLK, RCLK => RD_CLK, 
        RESET => RESETP, ESTOP => \VCC\, FSTOP => \VCC\, RD17 => 
        OPEN, RD16 => OPEN, RD15 => OPEN, RD14 => OPEN, RD13 => 
        OPEN, RD12 => OPEN, RD11 => OPEN, RD10 => OPEN, RD9 => 
        OPEN, RD8 => OPEN, RD7 => OPEN, RD6 => OPEN, RD5 => OPEN, 
        RD4 => OPEN, RD3 => OPEN, RD2 => OPEN, RD1 => OPEN, RD0
         => DOUT(5), FULL => \FULLX_I[5]\, AFULL => \AFULLX_I[5]\, 
        EMPTY => \EMPTYX_I[5]\, AEMPTY => \AEMPTYX_I[5]\);
    
    WRITE_AND : NAND2A
      port map(A => WEBP, B => WRITE_FSTOP_ENABLE, Y => 
        WRITE_ENABLE_I);
    
    OR2_16 : OR2
      port map(A => \EMPTYX_I[2]\, B => \EMPTYX_I[3]\, Y => 
        OR2_16_Y);
    
    OR2_12 : OR2
      port map(A => \EMPTYX_I[0]\, B => \EMPTYX_I[1]\, Y => 
        OR2_12_Y);
    
    OR2_19 : OR2
      port map(A => OR2_18_Y, B => OR2_11_Y, Y => OR2_19_Y);
    
    OR2_5 : OR2
      port map(A => OR2_10_Y, B => OR2_7_Y, Y => OR2_5_Y);
    
    OR2_FULL : OR2
      port map(A => OR2_20_Y, B => OR2_5_Y, Y => \FULL\);
    
    OR2_3 : OR2
      port map(A => \AFULLX_I[2]\, B => \AFULLX_I[3]\, Y => 
        OR2_3_Y);
    
    \FIFOBLOCK[2]\ : FIFO4K18
      port map(AEVAL11 => \GND\, AEVAL10 => \GND\, AEVAL9 => 
        \GND\, AEVAL8 => \GND\, AEVAL7 => \GND\, AEVAL6 => \GND\, 
        AEVAL5 => \GND\, AEVAL4 => \GND\, AEVAL3 => \VCC\, AEVAL2
         => \VCC\, AEVAL1 => \GND\, AEVAL0 => \VCC\, AFVAL11 => 
        \VCC\, AFVAL10 => \VCC\, AFVAL9 => \VCC\, AFVAL8 => \GND\, 
        AFVAL7 => \VCC\, AFVAL6 => \VCC\, AFVAL5 => \GND\, AFVAL4
         => \VCC\, AFVAL3 => \VCC\, AFVAL2 => \GND\, AFVAL1 => 
        \GND\, AFVAL0 => \GND\, WD17 => \GND\, WD16 => \GND\, 
        WD15 => \GND\, WD14 => \GND\, WD13 => \GND\, WD12 => 
        \GND\, WD11 => \GND\, WD10 => \GND\, WD9 => \GND\, WD8
         => \GND\, WD7 => \GND\, WD6 => \GND\, WD5 => \GND\, WD4
         => \GND\, WD3 => \GND\, WD2 => \GND\, WD1 => \GND\, WD0
         => DIN(2), WW0 => \GND\, WW1 => \GND\, WW2 => \GND\, RW0
         => \GND\, RW1 => \GND\, RW2 => \GND\, RPIPE => \GND\, 
        WEN => WRITE_ENABLE_I, REN => READ_ENABLE_I, WBLK => 
        \GND\, RBLK => \GND\, WCLK => WR_CLK, RCLK => RD_CLK, 
        RESET => RESETP, ESTOP => \VCC\, FSTOP => \VCC\, RD17 => 
        OPEN, RD16 => OPEN, RD15 => OPEN, RD14 => OPEN, RD13 => 
        OPEN, RD12 => OPEN, RD11 => OPEN, RD10 => OPEN, RD9 => 
        OPEN, RD8 => OPEN, RD7 => OPEN, RD6 => OPEN, RD5 => OPEN, 
        RD4 => OPEN, RD3 => OPEN, RD2 => OPEN, RD1 => OPEN, RD0
         => DOUT(2), FULL => \FULLX_I[2]\, AFULL => \AFULLX_I[2]\, 
        EMPTY => \EMPTYX_I[2]\, AEMPTY => \AEMPTYX_I[2]\);
    
    OR2_22 : OR2
      port map(A => \FULLX_I[0]\, B => \FULLX_I[1]\, Y => 
        OR2_22_Y);
    
    OR2_15 : OR2
      port map(A => OR2_9_Y, B => OR2_3_Y, Y => OR2_15_Y);
    
    OR2_4 : OR2
      port map(A => OR2_6_Y, B => OR2_21_Y, Y => OR2_4_Y);
    
    READ_ESTOP_GATE : NAND2
      port map(A => \EMPTY\, B => \VCC\, Y => READ_ESTOP_ENABLE);
    
    OR2_9 : OR2
      port map(A => \AFULLX_I[0]\, B => \AFULLX_I[1]\, Y => 
        OR2_9_Y);
    
    OR2_17 : OR2
      port map(A => \AEMPTYX_I[4]\, B => \AEMPTYX_I[5]\, Y => 
        OR2_17_Y);
    
    RESETBUBBLEA : INV
      port map(A => RST, Y => RESETP);
    
    OR2_14 : OR2
      port map(A => OR2_1_Y, B => OR2_13_Y, Y => OR2_14_Y);
    
    OR2_6 : OR2
      port map(A => \AFULLX_I[4]\, B => \AFULLX_I[5]\, Y => 
        OR2_6_Y);
    
    WRITE_FSTOP_GATE : NAND2
      port map(A => \FULL\, B => \VCC\, Y => WRITE_FSTOP_ENABLE);
    
    OR2_1 : OR2
      port map(A => \AEMPTYX_I[0]\, B => \AEMPTYX_I[1]\, Y => 
        OR2_1_Y);
    
    OR2_PROG_FULL : OR2
      port map(A => OR2_15_Y, B => OR2_4_Y, Y => PROG_FULL);
    
    OR2_8 : OR2
      port map(A => \FULLX_I[2]\, B => \FULLX_I[3]\, Y => OR2_8_Y);
    
    OR2_7 : OR2
      port map(A => \FULLX_I[6]\, B => \FULLX_I[7]\, Y => OR2_7_Y);
    
    \FIFOBLOCK[7]\ : FIFO4K18
      port map(AEVAL11 => \GND\, AEVAL10 => \GND\, AEVAL9 => 
        \GND\, AEVAL8 => \GND\, AEVAL7 => \GND\, AEVAL6 => \GND\, 
        AEVAL5 => \GND\, AEVAL4 => \GND\, AEVAL3 => \VCC\, AEVAL2
         => \VCC\, AEVAL1 => \GND\, AEVAL0 => \VCC\, AFVAL11 => 
        \VCC\, AFVAL10 => \VCC\, AFVAL9 => \VCC\, AFVAL8 => \GND\, 
        AFVAL7 => \VCC\, AFVAL6 => \VCC\, AFVAL5 => \GND\, AFVAL4
         => \VCC\, AFVAL3 => \VCC\, AFVAL2 => \GND\, AFVAL1 => 
        \GND\, AFVAL0 => \GND\, WD17 => \GND\, WD16 => \GND\, 
        WD15 => \GND\, WD14 => \GND\, WD13 => \GND\, WD12 => 
        \GND\, WD11 => \GND\, WD10 => \GND\, WD9 => \GND\, WD8
         => \GND\, WD7 => \GND\, WD6 => \GND\, WD5 => \GND\, WD4
         => \GND\, WD3 => \GND\, WD2 => \GND\, WD1 => \GND\, WD0
         => DIN(7), WW0 => \GND\, WW1 => \GND\, WW2 => \GND\, RW0
         => \GND\, RW1 => \GND\, RW2 => \GND\, RPIPE => \GND\, 
        WEN => WRITE_ENABLE_I, REN => READ_ENABLE_I, WBLK => 
        \GND\, RBLK => \GND\, WCLK => WR_CLK, RCLK => RD_CLK, 
        RESET => RESETP, ESTOP => \VCC\, FSTOP => \VCC\, RD17 => 
        OPEN, RD16 => OPEN, RD15 => OPEN, RD14 => OPEN, RD13 => 
        OPEN, RD12 => OPEN, RD11 => OPEN, RD10 => OPEN, RD9 => 
        OPEN, RD8 => OPEN, RD7 => OPEN, RD6 => OPEN, RD5 => OPEN, 
        RD4 => OPEN, RD3 => OPEN, RD2 => OPEN, RD1 => OPEN, RD0
         => DOUT(7), FULL => \FULLX_I[7]\, AFULL => \AFULLX_I[7]\, 
        EMPTY => \EMPTYX_I[7]\, AEMPTY => \AEMPTYX_I[7]\);
    
    READ_AND : AND2
      port map(A => RD_EN, B => READ_ESTOP_ENABLE, Y => 
        READ_ENABLE_I);
    
    OR2_10 : OR2
      port map(A => \FULLX_I[4]\, B => \FULLX_I[5]\, Y => 
        OR2_10_Y);
    
    \FIFOBLOCK[0]\ : FIFO4K18
      port map(AEVAL11 => \GND\, AEVAL10 => \GND\, AEVAL9 => 
        \GND\, AEVAL8 => \GND\, AEVAL7 => \GND\, AEVAL6 => \GND\, 
        AEVAL5 => \GND\, AEVAL4 => \GND\, AEVAL3 => \VCC\, AEVAL2
         => \VCC\, AEVAL1 => \GND\, AEVAL0 => \VCC\, AFVAL11 => 
        \VCC\, AFVAL10 => \VCC\, AFVAL9 => \VCC\, AFVAL8 => \GND\, 
        AFVAL7 => \VCC\, AFVAL6 => \VCC\, AFVAL5 => \GND\, AFVAL4
         => \VCC\, AFVAL3 => \VCC\, AFVAL2 => \GND\, AFVAL1 => 
        \GND\, AFVAL0 => \GND\, WD17 => \GND\, WD16 => \GND\, 
        WD15 => \GND\, WD14 => \GND\, WD13 => \GND\, WD12 => 
        \GND\, WD11 => \GND\, WD10 => \GND\, WD9 => \GND\, WD8
         => \GND\, WD7 => \GND\, WD6 => \GND\, WD5 => \GND\, WD4
         => \GND\, WD3 => \GND\, WD2 => \GND\, WD1 => \GND\, WD0
         => DIN(0), WW0 => \GND\, WW1 => \GND\, WW2 => \GND\, RW0
         => \GND\, RW1 => \GND\, RW2 => \GND\, RPIPE => \GND\, 
        WEN => WRITE_ENABLE_I, REN => READ_ENABLE_I, WBLK => 
        \GND\, RBLK => \GND\, WCLK => WR_CLK, RCLK => RD_CLK, 
        RESET => RESETP, ESTOP => \VCC\, FSTOP => \VCC\, RD17 => 
        OPEN, RD16 => OPEN, RD15 => OPEN, RD14 => OPEN, RD13 => 
        OPEN, RD12 => OPEN, RD11 => OPEN, RD10 => OPEN, RD9 => 
        OPEN, RD8 => OPEN, RD7 => OPEN, RD6 => OPEN, RD5 => OPEN, 
        RD4 => OPEN, RD3 => OPEN, RD2 => OPEN, RD1 => OPEN, RD0
         => DOUT(0), FULL => \FULLX_I[0]\, AFULL => \AFULLX_I[0]\, 
        EMPTY => \EMPTYX_I[0]\, AEMPTY => \AEMPTYX_I[0]\);
    
    \FIFOBLOCK[3]\ : FIFO4K18
      port map(AEVAL11 => \GND\, AEVAL10 => \GND\, AEVAL9 => 
        \GND\, AEVAL8 => \GND\, AEVAL7 => \GND\, AEVAL6 => \GND\, 
        AEVAL5 => \GND\, AEVAL4 => \GND\, AEVAL3 => \VCC\, AEVAL2
         => \VCC\, AEVAL1 => \GND\, AEVAL0 => \VCC\, AFVAL11 => 
        \VCC\, AFVAL10 => \VCC\, AFVAL9 => \VCC\, AFVAL8 => \GND\, 
        AFVAL7 => \VCC\, AFVAL6 => \VCC\, AFVAL5 => \GND\, AFVAL4
         => \VCC\, AFVAL3 => \VCC\, AFVAL2 => \GND\, AFVAL1 => 
        \GND\, AFVAL0 => \GND\, WD17 => \GND\, WD16 => \GND\, 
        WD15 => \GND\, WD14 => \GND\, WD13 => \GND\, WD12 => 
        \GND\, WD11 => \GND\, WD10 => \GND\, WD9 => \GND\, WD8
         => \GND\, WD7 => \GND\, WD6 => \GND\, WD5 => \GND\, WD4
         => \GND\, WD3 => \GND\, WD2 => \GND\, WD1 => \GND\, WD0
         => DIN(3), WW0 => \GND\, WW1 => \GND\, WW2 => \GND\, RW0
         => \GND\, RW1 => \GND\, RW2 => \GND\, RPIPE => \GND\, 
        WEN => WRITE_ENABLE_I, REN => READ_ENABLE_I, WBLK => 
        \GND\, RBLK => \GND\, WCLK => WR_CLK, RCLK => RD_CLK, 
        RESET => RESETP, ESTOP => \VCC\, FSTOP => \VCC\, RD17 => 
        OPEN, RD16 => OPEN, RD15 => OPEN, RD14 => OPEN, RD13 => 
        OPEN, RD12 => OPEN, RD11 => OPEN, RD10 => OPEN, RD9 => 
        OPEN, RD8 => OPEN, RD7 => OPEN, RD6 => OPEN, RD5 => OPEN, 
        RD4 => OPEN, RD3 => OPEN, RD2 => OPEN, RD1 => OPEN, RD0
         => DOUT(3), FULL => \FULLX_I[3]\, AFULL => \AFULLX_I[3]\, 
        EMPTY => \EMPTYX_I[3]\, AEMPTY => \AEMPTYX_I[3]\);
    
    \FIFOBLOCK[6]\ : FIFO4K18
      port map(AEVAL11 => \GND\, AEVAL10 => \GND\, AEVAL9 => 
        \GND\, AEVAL8 => \GND\, AEVAL7 => \GND\, AEVAL6 => \GND\, 
        AEVAL5 => \GND\, AEVAL4 => \GND\, AEVAL3 => \VCC\, AEVAL2
         => \VCC\, AEVAL1 => \GND\, AEVAL0 => \VCC\, AFVAL11 => 
        \VCC\, AFVAL10 => \VCC\, AFVAL9 => \VCC\, AFVAL8 => \GND\, 
        AFVAL7 => \VCC\, AFVAL6 => \VCC\, AFVAL5 => \GND\, AFVAL4
         => \VCC\, AFVAL3 => \VCC\, AFVAL2 => \GND\, AFVAL1 => 
        \GND\, AFVAL0 => \GND\, WD17 => \GND\, WD16 => \GND\, 
        WD15 => \GND\, WD14 => \GND\, WD13 => \GND\, WD12 => 
        \GND\, WD11 => \GND\, WD10 => \GND\, WD9 => \GND\, WD8
         => \GND\, WD7 => \GND\, WD6 => \GND\, WD5 => \GND\, WD4
         => \GND\, WD3 => \GND\, WD2 => \GND\, WD1 => \GND\, WD0
         => DIN(6), WW0 => \GND\, WW1 => \GND\, WW2 => \GND\, RW0
         => \GND\, RW1 => \GND\, RW2 => \GND\, RPIPE => \GND\, 
        WEN => WRITE_ENABLE_I, REN => READ_ENABLE_I, WBLK => 
        \GND\, RBLK => \GND\, WCLK => WR_CLK, RCLK => RD_CLK, 
        RESET => RESETP, ESTOP => \VCC\, FSTOP => \VCC\, RD17 => 
        OPEN, RD16 => OPEN, RD15 => OPEN, RD14 => OPEN, RD13 => 
        OPEN, RD12 => OPEN, RD11 => OPEN, RD10 => OPEN, RD9 => 
        OPEN, RD8 => OPEN, RD7 => OPEN, RD6 => OPEN, RD5 => OPEN, 
        RD4 => OPEN, RD3 => OPEN, RD2 => OPEN, RD1 => OPEN, RD0
         => DOUT(6), FULL => \FULLX_I[6]\, AFULL => \AFULLX_I[6]\, 
        EMPTY => \EMPTYX_I[6]\, AEMPTY => \AEMPTYX_I[6]\);
    
    OR2_2 : OR2
      port map(A => \AEMPTYX_I[6]\, B => \AEMPTYX_I[7]\, Y => 
        OR2_2_Y);
    
    \FIFOBLOCK[1]\ : FIFO4K18
      port map(AEVAL11 => \GND\, AEVAL10 => \GND\, AEVAL9 => 
        \GND\, AEVAL8 => \GND\, AEVAL7 => \GND\, AEVAL6 => \GND\, 
        AEVAL5 => \GND\, AEVAL4 => \GND\, AEVAL3 => \VCC\, AEVAL2
         => \VCC\, AEVAL1 => \GND\, AEVAL0 => \VCC\, AFVAL11 => 
        \VCC\, AFVAL10 => \VCC\, AFVAL9 => \VCC\, AFVAL8 => \GND\, 
        AFVAL7 => \VCC\, AFVAL6 => \VCC\, AFVAL5 => \GND\, AFVAL4
         => \VCC\, AFVAL3 => \VCC\, AFVAL2 => \GND\, AFVAL1 => 
        \GND\, AFVAL0 => \GND\, WD17 => \GND\, WD16 => \GND\, 
        WD15 => \GND\, WD14 => \GND\, WD13 => \GND\, WD12 => 
        \GND\, WD11 => \GND\, WD10 => \GND\, WD9 => \GND\, WD8
         => \GND\, WD7 => \GND\, WD6 => \GND\, WD5 => \GND\, WD4
         => \GND\, WD3 => \GND\, WD2 => \GND\, WD1 => \GND\, WD0
         => DIN(1), WW0 => \GND\, WW1 => \GND\, WW2 => \GND\, RW0
         => \GND\, RW1 => \GND\, RW2 => \GND\, RPIPE => \GND\, 
        WEN => WRITE_ENABLE_I, REN => READ_ENABLE_I, WBLK => 
        \GND\, RBLK => \GND\, WCLK => WR_CLK, RCLK => RD_CLK, 
        RESET => RESETP, ESTOP => \VCC\, FSTOP => \VCC\, RD17 => 
        OPEN, RD16 => OPEN, RD15 => OPEN, RD14 => OPEN, RD13 => 
        OPEN, RD12 => OPEN, RD11 => OPEN, RD10 => OPEN, RD9 => 
        OPEN, RD8 => OPEN, RD7 => OPEN, RD6 => OPEN, RD5 => OPEN, 
        RD4 => OPEN, RD3 => OPEN, RD2 => OPEN, RD1 => OPEN, RD0
         => DOUT(1), FULL => \FULLX_I[1]\, AFULL => \AFULLX_I[1]\, 
        EMPTY => \EMPTYX_I[1]\, AEMPTY => \AEMPTYX_I[1]\);
    
    OR2_PROG_EMPTY : OR2
      port map(A => OR2_14_Y, B => OR2_0_Y, Y => PROG_EMPTY);
    
    OR2_13 : OR2
      port map(A => \AEMPTYX_I[2]\, B => \AEMPTYX_I[3]\, Y => 
        OR2_13_Y);
    
    OR2_0 : OR2
      port map(A => OR2_17_Y, B => OR2_2_Y, Y => OR2_0_Y);
    
    OR2_20 : OR2
      port map(A => OR2_22_Y, B => OR2_8_Y, Y => OR2_20_Y);
    
    OR2_11 : OR2
      port map(A => \EMPTYX_I[6]\, B => \EMPTYX_I[7]\, Y => 
        OR2_11_Y);
    
    WEBUBBLEA : INV
      port map(A => WR_EN, Y => WEBP);
    
    OR2_18 : OR2
      port map(A => \EMPTYX_I[4]\, B => \EMPTYX_I[5]\, Y => 
        OR2_18_Y);
    
    OR2_23 : OR2
      port map(A => OR2_12_Y, B => OR2_16_Y, Y => OR2_23_Y);
    
    OR2_21 : OR2
      port map(A => \AFULLX_I[6]\, B => \AFULLX_I[7]\, Y => 
        OR2_21_Y);
    
    GND_power_inst1 : GND
      port map( Y => GND_power_net1);

    VCC_power_inst1 : VCC
      port map( Y => VCC_power_net1);


end DEF_ARCH; 

-- _Disclaimer: Please leave the following comments in the file, they are for internal purposes only._


-- _GEN_File_Contents_

-- Version:11.3.0.112
-- ACTGENU_CALL:1
-- BATCH:T
-- FAM:PA3LCLP
-- OUTFORMAT:VHDL
-- LPMTYPE:LPM_FIFO
-- LPM_HINT:NONE
-- INSERT_PAD:NO
-- INSERT_IOREG:NO
-- GEN_BHV_VHDL_VAL:F
-- GEN_BHV_VERILOG_VAL:F
-- MGNTIMER:F
-- MGNCMPL:T
-- DESDIR:C:/Users/ITP/Desktop/IGLOO/IGLOO_RHA/smartgen\FIFO
-- GEN_BEHV_MODULE:F
-- SMARTGEN_DIE:IS4X4M1LPLV
-- SMARTGEN_PACKAGE:vq100
-- AGENIII_IS_SUBPROJECT_LIBERO:T
-- WWIDTH:8
-- RWIDTH:8
-- WDEPTH:4096
-- RDEPTH:4096
-- WE_POLARITY:1
-- RE_POLARITY:1
-- RCLK_EDGE:RISE
-- WCLK_EDGE:RISE
-- PMODE1:0
-- FLAGS:STATIC
-- AFVAL:3800
-- AEVAL:13
-- ESTOP:NO
-- FSTOP:NO
-- AFVAL:3800
-- AEVAL:13
-- AFFLAG_UNITS:WW
-- AEFLAG_UNITS:RW
-- DATA_IN_PN:DIN
-- DATA_OUT_PN:DOUT
-- WE_PN:WR_EN
-- RE_PN:RD_EN
-- WCLOCK_PN:WR_CLK
-- RCLOCK_PN:RD_CLK
-- ACLR_PN:RST
-- FF_PN:FULL
-- EF_PN:EMPTY
-- AF_PN:PROG_FULL
-- AE_PN:PROG_EMPTY
-- AF_PORT_PN:AFVAL
-- AE_PORT_PN:AEVAL
-- RESET_POLARITY:1

-- _End_Comments_

