----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:50:33 12/30/2011 
-- Design Name: 
-- Module Name:    DataPacketRefurbish - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity DataPacketRefurbish is
    Port (
	 
			-- Connection to the FIFO (Zarlink Side)
			Data_In						: in  STD_LOGIC_VECTOR(143 downto 0);
			Valid_In						: in  STD_LOGIC;
			ReadEn_In					: out STD_LOGIC;
		
			-- Output Data (Network Side)
			NETWORK_FIFO_wr_en 		: out STD_LOGIC;
			NETWORK_FIFO_din 			: out STD_LOGIC_VECTOR(15 DOWNTO 0);
			NETWORK_FIFO_FULL			: in  STD_LOGIC;
		
			Implant_DeviceID 			: in  STD_LOGIC_VECTOR(7 downto 0);
			Bitmask 						: in  STD_LOGIC_VECTOR (15 downto 0);
			ChannelNumber 				: in  STD_LOGIC_VECTOR (15 downto 0);
			MaxCommandResponse		: in  STD_LOGIC_VECTOR (15 downto 0);
			
			Debug_DumpMode				: in  STD_LOGIC;
         Debug_EmptyMode         : in  STD_LOGIC;

         -- Secondary data path to the network interface 
         EXTERN_PlusFullPacket   : in  STD_LOGIC;
         EXTERN_FIFO_wr_en       : in  STD_LOGIC;
         EXTERN_FIFO_din         : in  STD_LOGIC_VECTOR(16 DOWNTO 0);
         EXTERN_FIFO_FULL        : out STD_LOGIC;

			-- The usual ports 
			Reset 						: in  STD_LOGIC;			
			Clk 							: in  STD_LOGIC
		);
end DataPacketRefurbish;

architecture Behavioral of DataPacketRefurbish is

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		STATE_HEADER_A,
		STATE_DATAMINING,
		STATE_PREPARE_HEADER,
		STATE_PREPARE_LENGTH,
		STATE_PREPARE_RESPONSEID,
		STATE_PREPARE_TIMESTAMP_A,
		STATE_PREPARE_TIMESTAMP_B,
		STATE_PREPARE_TRIGGER,
		STATE_PREPARE_DATA,
		STATE_PREPARE_NEW_WORD_CHECK,
		STATE_PREPARE_NEW_WORD,
		STATE_GET_NEW_SAMPLE,
		STATE_PREPARE_FILLUP_BITS,
		STATE_PREPARE_FILLUP,
		STATE_PREPARE_VALID,
		STATE_PACKAGE_DEFECT,
		STATE_PREPARE_HEADER_ERROR,
		STATE_PREPARE_LENGTH_ERROR,
		STATE_PREPARE_RESPONSEID_ERROR,
		STATE_PREPARE_ID_ERROR,
		STATE_PREPARE_ERROR_ERROR,
		STATE_COOLDOWN,
      STATE_NETWORK_COMMUNICATE_WAIT_A,
      STATE_NETWORK_COMMUNICATE_WAIT_B,
      STATE_NETWORK_COMMUNICATE_WAIT_C,
      STATE_NETWORK_COMMUNICATE_CRC00,
      STATE_NETWORK_COMMUNICATE_CRC01,
      STATE_NETWORK_COMMUNICATE_CRC10,
      STATE_NETWORK_COMMUNICATE_CRC11,
		STATE_DEBUG_DUMP_A,
		STATE_DEBUG_DUMP_B,
		STATE_DEBUG_DUMP_C,
		STATE_DEBUG_DUMP_D,
		STATE_DEBUG_DUMP_E,
		STATE_DEBUG_DUMP_F,
		STATE_DEBUG_DUMP_G,
		STATE_DEBUG_DUMP_H,
		STATE_DEBUG_DUMP_I,
		STATE_DEBUG_DUMP_J,
		STATE_DEBUG_DUMP_K,
		STATE_DEBUG_DUMP_L,
		STATE_DEBUG_DUMP_M,
		STATE_DEBUG_DUMP_N,
		STATE_DEBUG_DUMP_O,
		STATE_DEBUG_DUMP_P,
		STATE_DEBUG_DUMP_Q,
		STATE_DEBUG_DUMP_R,
		STATE_DEBUG_DUMP_S,
		STATE_DEBUG_DUMP_EXIT,
      STATE_EXTERN_COPY_A
	);
	signal FSM_STATE : STATE_TYPE;

	signal Data_In_Copy : STD_LOGIC_VECTOR(143 downto 0) := (others => '0');
	signal Data_Temp : STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	signal NumberOfSamples : unsigned (7 downto 0) := (others => '0');
	signal TriggerChannelsIn : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal The10KHzCounter : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal TimeStamp : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	-- First 4 bit
	constant PacketHeader_DATA : std_logic_vector(3 downto 0) := "0001";
	constant PacketHeader_CMDRESPONSE : std_logic_vector(3 downto 0) := "0011";

	-- 0x55AA
	constant NET_HEADER : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(21930,16));
	-- 0x8000
	constant NET_RESPONSEID_ERROR : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32768,16));		
	-- 0x8001
	constant NET_RESPONSEID_DATA : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32769,16));	
	-- 0x6542
	constant NET_RESPONSEID_CMD : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(25922,16));	

	signal ErrorState : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');

	-- Not constant because the length can change 
	signal NET_LENGTH_ERROR : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal NET_LENGTH_OK : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	signal HadToFillUp : std_logic := '0';
	signal DATA_PACKAGE_PROCESSING : std_logic := '0';


	signal ReadEn_In_Copy : STD_LOGIC := '0';
	signal Data_In_BitReverse	: STD_LOGIC_VECTOR(143 downto 0) := (others => '0');

	signal Bitmask_Copy 					: STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
	signal Bitmask_Temp 					: STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
	signal Bitmask_Index					: STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
	signal Sample_Temp					: STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
	
	signal ChannelNumber_Copy 				: STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
	signal MaxCommandResponse_Copy		: STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
	
	signal COUNTER_BITS_PROCESSED 		: unsigned (6 downto 0) := (others => '0');

   signal DEBUG_Counter                : unsigned (15 downto 0) := (others => '0');

   signal CRC_Reset_Internal : STD_LOGIC := '0';
   signal CRC_01_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
   signal CRC_10_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
   signal CRC_11_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');

   signal CRC_00 : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
   signal CRC_01 : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
   signal CRC_10 : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
   signal CRC_11 : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');

   signal EXTERN_CompletePacketInFIFO : STD_LOGIC := '0';
   signal EXTERN_MinusFullPacket : STD_LOGIC := '0';

   signal EXTERN_FIFO_rd_en : STD_LOGIC := '0';
   signal EXTERN_FIFO_dout : STD_LOGIC_VECTOR(16 DOWNTO 0) := (others => '0');
   signal EXTERN_FIFO_empty : STD_LOGIC := '0';
   signal EXTERN_FIFO_valid : STD_LOGIC := '0';

	signal FIFO_Error_Overrun : std_logic := '0';
	signal FIFO_Error_Underrun : std_logic := '0';
	
	signal FIFO_wr_en 				: STD_LOGIC := '0';
	signal FIFO_din 					: STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
	signal FIFO_FULL					: STD_LOGIC := '0';
	
begin
	-- This is the real data part of the packet 
	ReadEn_In <= ReadEn_In_Copy;
	NETWORK_FIFO_wr_en <= FIFO_wr_en;
	NETWORK_FIFO_din <= FIFO_din;
	FIFO_FULL <= NETWORK_FIFO_FULL;
	
	Data_In_BitReverse <= Data_In(143 downto 112)
		&Data_In(104)&Data_In(105)&Data_In(106)&Data_In(107)&Data_In(108)&Data_In(109)&Data_In(110)&Data_In(111)
		&Data_In(96)&Data_In(97)&Data_In(98)&Data_In(99)&Data_In(100)&Data_In(101)&Data_In(102)&Data_In(103)
		&Data_In(88)&Data_In(89)&Data_In(90)&Data_In(91)&Data_In(92)&Data_In(93)&Data_In(94)&Data_In(95)
		&Data_In(80)&Data_In(81)&Data_In(82)&Data_In(83)&Data_In(84)&Data_In(85)&Data_In(86)&Data_In(87)
		&Data_In(72)&Data_In(73)&Data_In(74)&Data_In(75)&Data_In(76)&Data_In(77)&Data_In(78)&Data_In(79)
		&Data_In(64)&Data_In(65)&Data_In(66)&Data_In(67)&Data_In(68)&Data_In(69)&Data_In(70)&Data_In(71)
		&Data_In(56)&Data_In(57)&Data_In(58)&Data_In(59)&Data_In(60)&Data_In(61)&Data_In(62)&Data_In(63)
		&Data_In(48)&Data_In(49)&Data_In(50)&Data_In(51)&Data_In(52)&Data_In(53)&Data_In(54)&Data_In(55)
		&Data_In(40)&Data_In(41)&Data_In(42)&Data_In(43)&Data_In(44)&Data_In(45)&Data_In(46)&Data_In(47)
		&Data_In(32)&Data_In(33)&Data_In(34)&Data_In(35)&Data_In(36)&Data_In(37)&Data_In(38)&Data_In(39)
		&Data_In(24)&Data_In(25)&Data_In(26)&Data_In(27)&Data_In(28)&Data_In(29)&Data_In(30)&Data_In(31)
		&Data_In(16)&Data_In(17)&Data_In(18)&Data_In(19)&Data_In(20)&Data_In(21)&Data_In(22)&Data_In(23)
		&Data_In(8)&Data_In(9)&Data_In(10)&Data_In(11)&Data_In(12)&Data_In(13)&Data_In(14)&Data_In(15)
		&Data_In(0)&Data_In(1)&Data_In(2)&Data_In(3)&Data_In(4)&Data_In(5)&Data_In(6)&Data_In(7);

	FSM : process (Reset, Clk)
	begin
		if Reset = '1' then
			FSM_STATE <= STATE_RESET;
			FIFO_wr_en <= '0';
			FIFO_din <= (others => '0');			
		
			ErrorState <= (others => '0');
			NET_LENGTH_ERROR <= std_logic_vector(to_unsigned(18,16));

			Bitmask_Copy <= (others => '0');
			Bitmask_Temp <= (others => '0');
			Bitmask_Index <= (others => '0');			
			
			ChannelNumber_Copy <= (others => '0');
			MaxCommandResponse_Copy <= (others => '0');

			Data_In_Copy <= (others => '0');
			NumberOfSamples <= (others => '0');
			ReadEn_In_Copy <= '0';
			HadToFillUp <= '0';
			
			DATA_PACKAGE_PROCESSING <= '0';
			
			TriggerChannelsIn <= (others => '0');
			The10KHzCounter <= (others => '0');
			TimeStamp <= (others => '0');
			NET_LENGTH_OK <= (others => '0');
			
			COUNTER_BITS_PROCESSED <= (others => '0');
			Data_Temp <= (others => '0');
			Sample_Temp <= (others => '0');
         DEBUG_Counter <= (others => '0');

         CRC_01_Copy <= (others => '0');
         CRC_10_Copy <= (others => '0');
         CRC_11_Copy <= (others => '0');
         CRC_Reset_Internal <= '0';

         EXTERN_MinusFullPacket <= '0';
         EXTERN_FIFO_rd_en <= '0';

		elsif rising_edge(Clk) then	
			
			-- Default:
			FSM_STATE <= STATE_RESET;
			FIFO_wr_en <= '0';
			FIFO_din <= (others => '0');			
			
			ErrorState <= ErrorState;
			NET_LENGTH_ERROR <= NET_LENGTH_ERROR;

			Bitmask_Copy <= Bitmask_Copy;
			Bitmask_Temp <= Bitmask_Temp;
			Bitmask_Index <= Bitmask_Index;		
			ChannelNumber_Copy <= ChannelNumber_Copy;
			MaxCommandResponse_Copy <= MaxCommandResponse_Copy;

			Data_In_Copy <= Data_In_Copy;
			NumberOfSamples <= NumberOfSamples;
			ReadEn_In_Copy <= '0';
			HadToFillUp <= HadToFillUp;

			DATA_PACKAGE_PROCESSING <= DATA_PACKAGE_PROCESSING;

			TriggerChannelsIn <= TriggerChannelsIn;
			The10KHzCounter <= The10KHzCounter;
			TimeStamp <= TimeStamp;
			NET_LENGTH_OK <= NET_LENGTH_OK;

			COUNTER_BITS_PROCESSED <= COUNTER_BITS_PROCESSED;
			Data_Temp <= Data_Temp;
			Sample_Temp <= Sample_Temp;
            
         DEBUG_Counter <= DEBUG_Counter;

         CRC_Reset_Internal <= '0';
         CRC_01_Copy <= (others => '0');
         CRC_10_Copy <= (others => '0');
         CRC_11_Copy <= (others => '0');

         EXTERN_MinusFullPacket <= '0';
         EXTERN_FIFO_rd_en <= '0';

			case FSM_STATE is
				when STATE_RESET =>
					FSM_STATE <= STATE_IDLE;
				
				when STATE_IDLE =>
					-- Is there valid data and is the number of samples known? 
					if  Valid_In = '1' then
						-- Yes => Let's start
						if Debug_DumpMode = '0' then
							FSM_STATE <= STATE_HEADER_A;
						else
							FSM_STATE <= STATE_DEBUG_DUMP_A; 
						end if;
						ReadEn_In_Copy <= '1';
               elsif EXTERN_CompletePacketInFIFO = '1' then
						FSM_STATE <= STATE_EXTERN_COPY_A; 
						EXTERN_MinusFullPacket <= '1';
					else
						-- No => Wait
						FSM_STATE <= STATE_IDLE;
					end if;
					ErrorState <= (others => '0');
					Data_In_Copy <= Data_In_BitReverse;
					NumberOfSamples <= (others => '0');
					HadToFillUp <= '0';
					ChannelNumber_Copy <= ChannelNumber;
					MaxCommandResponse_Copy <= MaxCommandResponse;

					
				when STATE_HEADER_A =>
				
					if Data_In_Copy(0) /= '1' then
						FSM_STATE <= STATE_COOLDOWN;
					elsif (PacketHeader_DATA        = Data_In_Copy(4 downto 1)) AND (Implant_DeviceID = Data_In_Copy(12 downto 5)) 
						AND ('0' = Data_In_Copy(30) ) then 
						FSM_STATE <= STATE_DATAMINING;
						DATA_PACKAGE_PROCESSING <= '1';
						Bitmask_Copy <= Bitmask;
						NET_LENGTH_OK <= std_logic_vector(to_unsigned(22,16)+unsigned(ChannelNumber_Copy)+unsigned(ChannelNumber_Copy));
					elsif (PacketHeader_CMDRESPONSE = Data_In_Copy(4 downto 1)) AND (Implant_DeviceID = Data_In_Copy(12 downto 5)) 
						AND ('0' = Data_In_Copy(30) ) then 
						FSM_STATE <= STATE_DATAMINING;
						DATA_PACKAGE_PROCESSING <= '0';
						Bitmask_Copy <= (others => '1');
						NET_LENGTH_OK <= std_logic_vector(to_unsigned(22,16)+unsigned(MaxCommandResponse_Copy)+unsigned(MaxCommandResponse_Copy));
					elsif PacketHeader_DATA = Data_In_Copy(4 downto 1)	then 
					-- Something wrong with the first part of a long header 
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						ErrorState <= std_logic_vector(to_unsigned(600,16));
					elsif PacketHeader_CMDRESPONSE = Data_In_Copy(4 downto 1)	then						
					-- Something wrong with the sort header 
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						ErrorState <= std_logic_vector(to_unsigned(601,16));
					elsif FIFO_Error_Overrun = '1' then
						-- If there was something wrong with the fifo then we have a problem
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						ErrorState <= std_logic_vector(to_unsigned(603,16));
					elsif FIFO_Error_Underrun = '1' then
						-- If there was something wrong with the fifo then we have a problem
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						ErrorState <= std_logic_vector(to_unsigned(604,16));						
					else
						-- Generate a defect package recieved network response !!!! 
						-- Unknown packet typ! 
						FSM_STATE <= STATE_PACKAGE_DEFECT;
						ErrorState <= std_logic_vector(to_unsigned(602,16));
					end if;
					TriggerChannelsIn <= Data_In_Copy(143 downto 128);
					The10KHzCounter <= Data_In_Copy(127 downto 112);
					Timestamp <= Data_In_Copy(28 downto 13);
               CRC_Reset_Internal <= '1';

				-- Wait until there is space in the FIFO... then begin!
				-- NOTE: Packet is closed with FIDO(16) => '1' at it's last element
				-- CRC is calculated later but add the 8 byte to the length!
				when STATE_DATAMINING => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DATAMINING;
					else
						FSM_STATE <= STATE_PREPARE_HEADER;
					end if;
					Bitmask_Temp <= Bitmask_Copy;
					Bitmask_Index <= (others => '1');
					Sample_Temp <= (others => '0');
					
				when STATE_PREPARE_HEADER => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_HEADER;
               else
						FIFO_wr_en <= '1';
						FIFO_din <= NET_HEADER;				
						FSM_STATE <= STATE_PREPARE_LENGTH;
               end if;
					
				when STATE_PREPARE_LENGTH =>
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_LENGTH;  
               else
						FIFO_wr_en <= '1';
						FIFO_din <= NET_LENGTH_OK;
						FSM_STATE <= STATE_PREPARE_RESPONSEID;					
               end if;

				when STATE_PREPARE_RESPONSEID => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_RESPONSEID;					
               else
						FIFO_wr_en <= '1';
						if DATA_PACKAGE_PROCESSING = '1' then
							FIFO_din <= NET_RESPONSEID_DATA;
						else
							FIFO_din <= NET_RESPONSEID_CMD;
						end if;
						FSM_STATE <= STATE_PREPARE_TIMESTAMP_A;
               end if;
					
				when STATE_PREPARE_TIMESTAMP_A =>
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_TIMESTAMP_A;
               else
						FIFO_wr_en <= '1';
						FIFO_din <= TimeStamp;
						FSM_STATE <= STATE_PREPARE_TIMESTAMP_B;
					end if;

				when STATE_PREPARE_TIMESTAMP_B =>
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_TIMESTAMP_B;
               else
						FIFO_wr_en <= '1';
						FIFO_din <= The10KHzCounter;
						FSM_STATE <= STATE_PREPARE_TRIGGER;
					end if;

				when STATE_PREPARE_TRIGGER =>
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_TRIGGER;
               else
						FIFO_wr_en <= '1';
						FIFO_din <= TriggerChannelsIn;				
						FSM_STATE <= STATE_PREPARE_DATA;
               end if;

				when STATE_PREPARE_DATA =>
					-- Reset the data processor 
					if (DATA_PACKAGE_PROCESSING = '1') AND (NumberOfSamples >= unsigned(ChannelNumber_Copy)) then
						-- Done... 
						FSM_STATE <= STATE_PREPARE_FILLUP;
					elsif (DATA_PACKAGE_PROCESSING = '0') AND (NumberOfSamples >= unsigned(MaxCommandResponse_Copy)) then
						-- Done... 
						FSM_STATE <= STATE_PREPARE_FILLUP;				
					else
						FSM_STATE <= STATE_PREPARE_NEW_WORD;
					end if;
					
					if Data_In_Copy(0) = '1' then
						COUNTER_BITS_PROCESSED <= to_unsigned(31,7);
						Data_Temp <= "0" & "0000000000" & "0000000000" & "0000000000" & Data_In_Copy(111 downto 31);
					else
						COUNTER_BITS_PROCESSED <= to_unsigned( 1,7);
						Data_Temp <= "0" & Data_In_Copy(111 downto 1);
					end if;


				when STATE_PREPARE_NEW_WORD_CHECK => 
					-- Stop at the maximum number of expected samples 
					-- Don't send more than that 
					if (DATA_PACKAGE_PROCESSING = '1') AND (NumberOfSamples >= unsigned(ChannelNumber_Copy)) then
						-- Done... 
						FSM_STATE <= STATE_PREPARE_FILLUP;
					elsif (DATA_PACKAGE_PROCESSING = '0') AND (NumberOfSamples >= unsigned(MaxCommandResponse_Copy)) then
						-- Done... 
						FSM_STATE <= STATE_PREPARE_FILLUP;				
					else
						FSM_STATE <= STATE_PREPARE_NEW_WORD;
					end if;
					
				when STATE_PREPARE_NEW_WORD => 

					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_NEW_WORD;
               elsif Bitmask_Index(0) = '0' then
						FSM_STATE <= STATE_PREPARE_NEW_WORD_CHECK;
						FIFO_din <= Sample_Temp;
						FIFO_wr_en <= '1';	
						Bitmask_Temp <= Bitmask_Copy;
						Bitmask_Index <= (others => '1');
						Sample_Temp <= (others => '0');
						NumberOfSamples <= NumberOfSamples + 1;
					elsif COUNTER_BITS_PROCESSED = to_unsigned(112,7) then 
						FSM_STATE <= STATE_GET_NEW_SAMPLE;
					else
						FSM_STATE <= STATE_PREPARE_NEW_WORD;
						Bitmask_Temp <= "0" & Bitmask_Temp(15 downto 1);
						Bitmask_Index <= "0" & Bitmask_Index(15 downto 1);
						Data_Temp <= "0" & Data_Temp(111 downto 1);
						COUNTER_BITS_PROCESSED <= COUNTER_BITS_PROCESSED + 1;
						
						if Bitmask_Temp(0) = '1' then
						--	Sample_Temp <= Data_Temp(0) & Sample_Temp(15 downto 1);
							Sample_Temp <= Sample_Temp(14 downto 0) & Data_Temp(0);
						else
						--	Sample_Temp <= "0" & Sample_Temp(15 downto 1);
							Sample_Temp <= Sample_Temp(14 downto 0) & "0";
						end if;						
					end if;

				when STATE_GET_NEW_SAMPLE => 

					if (Valid_In = '1') AND (Data_In_BitReverse(0) = '1') then
						-- This can be bad!!! 
						-- Packet could have ended to soon... 
						FSM_STATE <= STATE_PREPARE_FILLUP_BITS;
					elsif (DATA_PACKAGE_PROCESSING = '1') AND (NumberOfSamples >= unsigned(ChannelNumber_Copy)) then
						-- Done... 
						FSM_STATE <= STATE_PREPARE_FILLUP_BITS;
					elsif (DATA_PACKAGE_PROCESSING = '0') AND (NumberOfSamples >= unsigned(MaxCommandResponse_Copy)) then
						-- Done... 
						FSM_STATE <= STATE_PREPARE_FILLUP_BITS;
					elsif (Valid_In = '1') AND (ReadEn_In_Copy = '0') then
						-- Looks good... 
						ReadEn_In_Copy <= '1';
						Data_In_Copy <= Data_In_BitReverse;
						FSM_STATE <= STATE_PREPARE_DATA;
					else
						-- No Data here yet
						FSM_STATE <= STATE_GET_NEW_SAMPLE;
					end if;

				when STATE_PREPARE_FILLUP_BITS => 

					if FIFO_FULL = '1' then
                  FSM_STATE <= STATE_PREPARE_FILLUP_BITS;
					elsif Bitmask_Index(0) = '0' then
						FSM_STATE <= STATE_PREPARE_FILLUP;
						FIFO_din <= Sample_Temp;
						FIFO_wr_en <= '1';		
						Bitmask_Temp <= Bitmask_Copy;
						Bitmask_Index <= (others => '1');
						Sample_Temp <= (others => '0');						
						NumberOfSamples <= NumberOfSamples + 1;
					else
						FSM_STATE <= STATE_PREPARE_FILLUP_BITS;
						Bitmask_Index <= "0" & Bitmask_Index(15 downto 1);
						--Sample_Temp <= "0" & Sample_Temp(15 downto 1);
						Sample_Temp <= Sample_Temp(14 downto 0) & "0";
					end if;

				when STATE_PREPARE_FILLUP =>
				
					if FIFO_FULL = '1' then
                  FSM_STATE <= STATE_PREPARE_FILLUP;
					elsif (DATA_PACKAGE_PROCESSING = '1') AND (NumberOfSamples < unsigned(ChannelNumber_Copy)) then
						FIFO_wr_en <= '1';
						FIFO_din <= (others => '0');
						NumberOfSamples <= NumberOfSamples + 1;
						HadToFillUp <= '1';
						FSM_STATE <= STATE_PREPARE_FILLUP;
					elsif (DATA_PACKAGE_PROCESSING = '0') AND (NumberOfSamples < unsigned(MaxCommandResponse_Copy)) then
						FIFO_wr_en <= '1';
						FIFO_din <= (others => '0');
						NumberOfSamples <= NumberOfSamples + 1;
						HadToFillUp <= '0';
						FSM_STATE <= STATE_PREPARE_FILLUP;
					else
						FSM_STATE <= STATE_PREPARE_VALID;
					end if;
					
				when STATE_PREPARE_VALID =>
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_VALID;
               else
						FIFO_wr_en <= '1';
					
						-- sufficent samples 
						if HadToFillUp = '0' then
							FIFO_din <= "0000000000000001";				
						else
						-- too few samples
							FIFO_din <= "0000000000000010";				
						end if;
					
						FSM_STATE <= STATE_COOLDOWN;
					end if;

---------------------------------------------------------------------------------------------------

					when STATE_COOLDOWN =>
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;

                -- Wait for the CRC - Values I.
               when STATE_NETWORK_COMMUNICATE_WAIT_A =>
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_B;

                -- Wait for the CRC - Values II.
               when STATE_NETWORK_COMMUNICATE_WAIT_B =>
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_C;

                    -- Wait for the CRC - Values III.
               when STATE_NETWORK_COMMUNICATE_WAIT_C =>
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;

                    -- Communicate with the network (CRC 00)
               when STATE_NETWORK_COMMUNICATE_CRC00 =>
							CRC_01_Copy <= CRC_01;
							CRC_10_Copy <= CRC_10;
							CRC_11_Copy <= CRC_11;

							if FIFO_FULL = '1' then
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
							else
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
								FIFO_wr_en <= '1';
								FIFO_din <= CRC_00;				
                    end if;

                -- Communicate with the network (CRC 01)
                when STATE_NETWORK_COMMUNICATE_CRC01 =>
							CRC_01_Copy <= CRC_01_Copy;
							CRC_10_Copy <= CRC_10_Copy;
							CRC_11_Copy <= CRC_11_Copy;

							if FIFO_FULL = '1' then
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
							else
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
								FIFO_wr_en <= '1';
								FIFO_din <= CRC_01_Copy;				
							end if;

                -- Communicate with the network (CRC 10)
                when STATE_NETWORK_COMMUNICATE_CRC10 =>
							CRC_10_Copy <= CRC_10_Copy;
							CRC_11_Copy <= CRC_11_Copy;

							if FIFO_FULL = '1' then
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
							else
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
								FIFO_wr_en <= '1';
								FIFO_din <= CRC_10_Copy;				
							end if;

                -- Communicate with the network (CRC 11)
                when STATE_NETWORK_COMMUNICATE_CRC11 =>
							CRC_11_Copy <= CRC_11_Copy;

							if FIFO_FULL = '1' then
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
							else
								FSM_STATE <= STATE_IDLE;
								FIFO_wr_en <= '1';
								FIFO_din <= CRC_11_Copy;				
							end if;


-----------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------
----- ERROR HANDLING: 	
			
				when STATE_PACKAGE_DEFECT => 

					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PACKAGE_DEFECT;
					else
						FSM_STATE <= STATE_PREPARE_HEADER_ERROR;
					end if;

				when STATE_PREPARE_HEADER_ERROR => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_HEADER_ERROR;
               else
						FIFO_wr_en <= '1';
						FIFO_din <= NET_HEADER;				
						FSM_STATE <= STATE_PREPARE_LENGTH_ERROR;
					end if;

				when STATE_PREPARE_LENGTH_ERROR =>
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_LENGTH_ERROR;
               else
						FIFO_wr_en <= '1';
						FIFO_din <= NET_LENGTH_ERROR;
						FSM_STATE <= STATE_PREPARE_RESPONSEID_ERROR;
					end if;

				when STATE_PREPARE_RESPONSEID_ERROR =>
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_RESPONSEID_ERROR;
               else
						FIFO_wr_en <= '1';
						FIFO_din <= NET_RESPONSEID_ERROR;
						FSM_STATE <= STATE_PREPARE_ID_ERROR;
					end if;

				when STATE_PREPARE_ID_ERROR =>
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_PREPARE_ID_ERROR;
               else
						FIFO_wr_en <= '1';
						FIFO_din <= std_logic_vector(to_unsigned(254,16));
						FSM_STATE <= STATE_PREPARE_ERROR_ERROR;
					end if;

				when STATE_PREPARE_ERROR_ERROR =>				
					if FIFO_FULL = '1' then
                  FSM_STATE <= STATE_PREPARE_ERROR_ERROR;
				   else	
						FIFO_wr_en <= '1';
						FIFO_din <= ErrorState;				
						FSM_STATE <= STATE_COOLDOWN;
               end if;
				
--------- DEBUG.... 

				when STATE_DEBUG_DUMP_A => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_A;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
						if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
						else
							FIFO_din <= "0000000011111111";
						end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_B;
					end if;
					
				when STATE_DEBUG_DUMP_B => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_B;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= "0000001011111111";
                  end if;

						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_C;
					end if;					

				when STATE_DEBUG_DUMP_C => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_C;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Bitmask;
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_D;
					end if;

				when STATE_DEBUG_DUMP_D => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_D;
					else
                  DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= "0000011011111111";
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_E;
					end if;					

				when STATE_DEBUG_DUMP_E => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_E;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= ChannelNumber;
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_F;
					end if;

				when STATE_DEBUG_DUMP_F => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_F;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= "0000111011111111";
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_G;
					end if;	

				when STATE_DEBUG_DUMP_G => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_G;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
						if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= MaxCommandResponse;
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_H;
					end if;

				when STATE_DEBUG_DUMP_H => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_H;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= "0001111011111111";
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_I;
					end if;	

				when STATE_DEBUG_DUMP_I => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_I;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= "00000000" & Implant_DeviceID;
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_J;
					end if;

				when STATE_DEBUG_DUMP_J => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_J;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= "0011111011111111";
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_K;
					end if;

				when STATE_DEBUG_DUMP_K => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_K;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(15 downto 0);
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_L;
					end if;

				when STATE_DEBUG_DUMP_L => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_L;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
						if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(31 downto 16);
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_M;
					end if;

				when STATE_DEBUG_DUMP_M => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_M;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(47 downto 32);
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_N;
					end if;

				when STATE_DEBUG_DUMP_N => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_N;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(63 downto 48);
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_O;
					end if;

				when STATE_DEBUG_DUMP_O => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_O;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(79 downto 64);
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_P;
					end if;

				when STATE_DEBUG_DUMP_P => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_P;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(95 downto 80);
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_Q;
					end if;

				when STATE_DEBUG_DUMP_Q => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_Q;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
						if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(111 downto 96);
						end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_R;
					end if;

				when STATE_DEBUG_DUMP_R => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_R;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(127 downto 112);
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_S;
					end if;

				when STATE_DEBUG_DUMP_S => 
					if FIFO_FULL = '1' then
						FSM_STATE <= STATE_DEBUG_DUMP_S;
					else
						DEBUG_Counter <= DEBUG_Counter + 1;
                  if Debug_EmptyMode = '1' then
							FIFO_din <= std_logic_vector(DEBUG_Counter);
                  else
							FIFO_din <= Data_In_Copy(143 downto 128);
                  end if;
						FIFO_wr_en <= '1';
						FSM_STATE <= STATE_DEBUG_DUMP_EXIT;
					end if;
				
					when STATE_DEBUG_DUMP_EXIT => 
						FSM_STATE <= STATE_IDLE;

                -- Copy the data from the secondary data path to the network interface 
                when STATE_EXTERN_COPY_A => 
                  if FIFO_FULL = '1' then
                      FSM_STATE <= STATE_EXTERN_COPY_A;
                  elsif (EXTERN_FIFO_valid = '1') AND (EXTERN_FIFO_dout(16) = '1') then
                      FIFO_wr_en <= '1';
                      FIFO_din <= EXTERN_FIFO_dout(15 downto 0);
                      FSM_STATE <= STATE_IDLE;
							 EXTERN_FIFO_rd_en <= '1';
                  elsif (EXTERN_FIFO_valid = '1') AND (EXTERN_FIFO_dout(16) = '0') AND (EXTERN_FIFO_rd_en = '0') then
                      EXTERN_FIFO_rd_en <= '1';
                      FSM_STATE <= STATE_EXTERN_COPY_A;
                  elsif (EXTERN_FIFO_valid = '1') AND (EXTERN_FIFO_dout(16) = '0') AND (EXTERN_FIFO_rd_en = '1') then
                      FIFO_wr_en <= '1';
                      FIFO_din <= EXTERN_FIFO_dout(15 downto 0);
                      FSM_STATE <= STATE_EXTERN_COPY_A;
                  else
                      FSM_STATE <= STATE_EXTERN_COPY_A;                  
                  end if;

				when others =>
					FSM_STATE <= STATE_RESET;
			end case;
		end if;
	end process FSM;

    MyCRC : entity work.CRC_Checker
    Port MAP(
          Network_Data => FIFO_din,
          Network_Valid => FIFO_wr_en,
          CRC_Reset => CRC_Reset_INTERNAL,
          CRC_00 => CRC_00,
          CRC_01 => CRC_01,
          CRC_10 => CRC_10,
          CRC_11 => CRC_11,
          Reset => Reset,
          Clk => Clk
    );

    MyNetworkPacketCounter : entity work.NetworkPacketCounter
    Port Map(
        CompletePacketInFIFO => EXTERN_CompletePacketInFIFO,
        Plus => EXTERN_PlusFullPacket,
        Minus => EXTERN_MinusFullPacket,

        Error_Overrun => FIFO_Error_Overrun,
        Error_Underrun => FIFO_Error_Underrun,
        Reset => Reset,
        Clk => Clk
    );

    MyNetworkOutgoingPaketProcessingFIFO : entity work.NetworkOutgoingPaketProcessingFIFO
    PORT MAP (
      clk => Clk,
      rst => Reset,
      din => EXTERN_FIFO_din,
      wr_en => EXTERN_FIFO_wr_en,
      rd_en => EXTERN_FIFO_rd_en,    
      dout => EXTERN_FIFO_dout,
      full => open,
      almost_full => EXTERN_FIFO_FULL,
      empty => open,
      valid => EXTERN_FIFO_valid,
      prog_full => EXTERN_FIFO_empty
  );


end Behavioral;


