library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

ENTITY DataPacketRefurbish_UUT IS
END DataPacketRefurbish_UUT;
 
ARCHITECTURE behavior OF DataPacketRefurbish_UUT IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT DataPacketRefurbish
    PORT(
         Data_In : IN  std_logic_vector(143 downto 0);
         Valid_In : IN  std_logic;
         ReadEn_In : OUT  std_logic;
         FIFO_PlusFullPacket : OUT  std_logic;
         FIFO_wr_en : OUT  std_logic;
         FIFO_din : OUT  std_logic_vector(16 downto 0);
         FIFO_FULL : IN  std_logic;
         FIFO_Error_Overrun : IN  std_logic;
         FIFO_Error_Underrun : IN  std_logic;
         Implant_DeviceID : IN  std_logic_vector(7 downto 0);
         Bitmask : IN  std_logic_vector(15 downto 0);
         ChannelNumber : IN  std_logic_vector(15 downto 0);
         MaxCommandResponse : IN  std_logic_vector(15 downto 0);
         Debug_DumpMode : IN  std_logic;
         Reset : IN  std_logic;
         Clk : IN  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal Data_In : std_logic_vector(143 downto 0) := (others => '0');
   signal Valid_In : std_logic := '0';
   signal FIFO_FULL : std_logic := '0';
   signal FIFO_Error_Overrun : std_logic := '0';
   signal FIFO_Error_Underrun : std_logic := '0';
   signal Implant_DeviceID : std_logic_vector(7 downto 0) := (others => '0');
   signal BitNumber : std_logic_vector(15 downto 0) := (others => '0');
   signal ChannelNumber : std_logic_vector(15 downto 0) := (others => '0');
   signal MaxCommandResponse : std_logic_vector(15 downto 0) := (others => '0');
   signal Debug_DumpMode : std_logic := '0';
   signal Reset : std_logic := '0';
   signal Clk : std_logic := '0';

 	--Outputs
   signal ReadEn_In : std_logic;
   signal FIFO_PlusFullPacket : std_logic;
   signal FIFO_wr_en : std_logic;
   signal FIFO_din : std_logic_vector(16 downto 0);

   -- Clock period definitions
   constant Clk_period : time := 10 ns;

  	signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
	signal InputCounter : unsigned(127 downto 0) := (others=>'0'); 
 	signal Data_In_BitReverse	: STD_LOGIC_VECTOR(143 downto 0) := (others => '0');

	constant PacketHeader_DATA : std_logic_vector(3 downto 0) := "0001";
	constant PacketHeader_CMDRESPONSE : std_logic_vector(3 downto 0) := "0011";
	constant Implant_DeviceID_Buffer : std_logic_vector(7 downto 0) := std_logic_vector(to_unsigned(53,8));	
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: DataPacketRefurbish PORT MAP (
          Data_In => Data_In_BitReverse,
          Valid_In => Valid_In,
          ReadEn_In => ReadEn_In,
          FIFO_PlusFullPacket => FIFO_PlusFullPacket,
          FIFO_wr_en => FIFO_wr_en,
          FIFO_din => FIFO_din,
          FIFO_FULL => FIFO_FULL,
          FIFO_Error_Overrun => FIFO_Error_Overrun,
          FIFO_Error_Underrun => FIFO_Error_Underrun,
          Implant_DeviceID => Implant_DeviceID_Buffer,
          Bitmask => "1111111111111111",
          ChannelNumber => std_logic_vector(to_unsigned(3,16)),
          MaxCommandResponse => std_logic_vector(to_unsigned(9,16)),
          Debug_DumpMode => Debug_DumpMode,
          Reset => Reset,
          Clk => Clk
        );

 TheMasterClock : process
begin
	clockloop: for i in 0 to 500000 loop
		clk <= NOT clk;
		wait for 4ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 


	Data_In_BitReverse <= Data_In(143 downto 112)
		&Data_In(104)&Data_In(105)&Data_In(106)&Data_In(107)&Data_In(108)&Data_In(109)&Data_In(110)&Data_In(111)
		&Data_In(96)&Data_In(97)&Data_In(98)&Data_In(99)&Data_In(100)&Data_In(101)&Data_In(102)&Data_In(103)
		&Data_In(88)&Data_In(89)&Data_In(90)&Data_In(91)&Data_In(92)&Data_In(93)&Data_In(94)&Data_In(95)
		&Data_In(80)&Data_In(81)&Data_In(82)&Data_In(83)&Data_In(84)&Data_In(85)&Data_In(86)&Data_In(87)
		&Data_In(72)&Data_In(73)&Data_In(74)&Data_In(75)&Data_In(76)&Data_In(77)&Data_In(78)&Data_In(79)
		&Data_In(64)&Data_In(65)&Data_In(66)&Data_In(67)&Data_In(68)&Data_In(69)&Data_In(70)&Data_In(71)
		&Data_In(56)&Data_In(57)&Data_In(58)&Data_In(59)&Data_In(60)&Data_In(61)&Data_In(62)&Data_In(63)
		&Data_In(48)&Data_In(49)&Data_In(50)&Data_In(51)&Data_In(52)&Data_In(53)&Data_In(54)&Data_In(55)
		&Data_In(40)&Data_In(41)&Data_In(42)&Data_In(43)&Data_In(44)&Data_In(45)&Data_In(46)&Data_In(47)
		&Data_In(32)&Data_In(33)&Data_In(34)&Data_In(35)&Data_In(36)&Data_In(37)&Data_In(38)&Data_In(39)
		&Data_In(24)&Data_In(25)&Data_In(26)&Data_In(27)&Data_In(28)&Data_In(29)&Data_In(30)&Data_In(31)
		&Data_In(16)&Data_In(17)&Data_In(18)&Data_In(19)&Data_In(20)&Data_In(21)&Data_In(22)&Data_In(23)
		&Data_In(8)&Data_In(9)&Data_In(10)&Data_In(11)&Data_In(12)&Data_In(13)&Data_In(14)&Data_In(15)
		&Data_In(0)&Data_In(1)&Data_In(2)&Data_In(3)&Data_In(4)&Data_In(5)&Data_In(6)&Data_In(7);

TheSecondClock : process(clk) 
begin
	if rising_edge(clk) then
		
		MasterCounter <= MasterCounter + 1;
		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;

--	PacketHeader_DATA;
		
		if ReadEn_In = '1' then
			InputCounter <= InputCounter + 1;
		else
			InputCounter <= InputCounter;
		end if;
		
		if InputCounter = to_unsigned(0,128) then
			Data_In <= (others => '0');
			Data_In(0) <= '1';
			Data_In(4 downto 1) <= PacketHeader_CMDRESPONSE;
			Data_In(12 downto 5) <= Implant_DeviceID_Buffer;
			Data_In(30) <= '0';
			Valid_In <= '1';
		elsif InputCounter = to_unsigned(1,128) then
			Data_In <= (others => '1');
			Data_In(0) <= '1';
			Data_In(4 downto 1) <= PacketHeader_CMDRESPONSE;
			Data_In(12 downto 5) <= Implant_DeviceID_Buffer;
			Data_In(30) <= '0';			
			Valid_In <= '1';
		elsif InputCounter = to_unsigned(2,128) then
			Data_In <= (others => '1');
			Data_In(0) <= '0';
			Valid_In <= '1';			
		else
			Valid_In <= '0';
			Data_In <= (others => '0');
		end if;
		
	end if;
end process TheSecondClock;
END;
