-- Written by David Rotermund (davrot@neuro.uni-bremen.de)
-- University of Bremen, Institute for theoretical physiscs
-- (Use Atom.IO editor for correct indents)

-- Top module for
-- Targeted device: <Family::IGLOO> <Die::AGL250V2> <Package::100 VQFP>

-- Version 0.0 (23.07.14) : Original source code was written
-- Version 0.1 (03.07.15) : Added debug info, reformating with Atom.IO

library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity IGLOO_TOP is
  port (
  -- Connection to the Zarlink
    SPI_ZL_CS           : out STD_LOGIC;
    SPI_SDI             : out STD_LOGIC;
    SPI_SDO             :  in STD_LOGIC;
    SPI_CLK             : out STD_LOGIC;

    RHD_MOSI            : out STD_LOGIC_VECTOR(7 downto 0);
    RHD_MISO            :  in STD_LOGIC_VECTOR(7 downto 0);
    RHD_CLK             : out STD_LOGIC_VECTOR(7 downto 0);
    RHD_CS              : out STD_LOGIC_VECTOR(7 downto 0);

    -- DEBUG SECTION
    DEBUG_STATE         : out std_logic_vector(8 downto 0);
    DEBUG_SELECTOR_STATE:  in std_logic_vector(4 downto 0);

    DEBUG_CLOCK         : out std_logic;
    DEBUG_SELECTOR_CLOCK:  in std_logic_vector(1 downto 0);

    Reset               :  in STD_LOGIC;
    Clk                 :  in STD_LOGIC
);
end IGLOO_TOP;
architecture IGLOO_TOP_ARCH of IGLOO_TOP is

  signal DEBUG_S_RHDTES_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_RHDTES_FSM1_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_RHD2ZL_FSM0_STATE         : std_logic_vector(3 downto 0) := (others => '0');
  signal DEBUG_S_RHDARR_FSM0_STATE         : std_logic_vector(3 downto 0) := (others => '0');
  signal DEBUG_S_DACOL0_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_DACOL1_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_DACOL2_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_DACOL3_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_DACOL4_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_DACOL5_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_DACOL6_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_DACOL7_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_CLOCK_ORIGINAL              : std_logic := '0';
  signal DEBUG_CLOCK_HALF                  : std_logic := '0';
  signal DEBUG_CLOCK_4MHz                  : std_logic := '0';
  signal DEBUG_S_COMREC_FSM0_STATE         : std_logic_vector(4 downto 0) := (others => '0');
  signal DEBUG_S_ZARINI_FSM0_STATE         : std_logic_vector(5 downto 0) := (others => '0');
  signal DEBUG_S_ORGANI_FSM0_STATE         : std_logic_vector(3 downto 0) := (others => '0');
  signal DEBUG_S_ZARSPI_FSM0_STATE         : std_logic_vector(5 downto 0) := (others => '0');
  signal DEBUG_P_DAACBL_1                  : std_logic_vector(8 downto 0) := (others => '0');
  signal DEBUG_P_DAACBL_2                  : std_logic_vector(8 downto 0) := (others => '0');

  signal Reset_NOT                         : std_logic := '0';
  constant WaitForReset                     : unsigned(23 downto 0) := to_unsigned(12000000,24);
  signal ResetCounter                       : unsigned(23 downto 0) := (others => '0');
begin

  -- DEBUGGING FSM
  DEBUG_STATE_PROCESS : process (DEBUG_SELECTOR_STATE,
   DEBUG_S_RHDTES_FSM0_STATE, DEBUG_S_RHDTES_FSM1_STATE, DEBUG_S_RHD2ZL_FSM0_STATE,
   DEBUG_S_RHDARR_FSM0_STATE, DEBUG_S_DACOL0_FSM0_STATE, DEBUG_S_DACOL1_FSM0_STATE,
   DEBUG_S_DACOL2_FSM0_STATE, DEBUG_S_DACOL3_FSM0_STATE, DEBUG_S_DACOL4_FSM0_STATE,
   DEBUG_S_DACOL5_FSM0_STATE, DEBUG_S_DACOL6_FSM0_STATE, DEBUG_S_DACOL7_FSM0_STATE,
   DEBUG_S_COMREC_FSM0_STATE, DEBUG_S_ZARINI_FSM0_STATE, DEBUG_S_ORGANI_FSM0_STATE,
   DEBUG_S_ZARSPI_FSM0_STATE, DEBUG_P_DAACBL_1, DEBUG_P_DAACBL_2)
  begin
    case DEBUG_SELECTOR_STATE is
      when "00000" =>
        DEBUG_STATE <= "0000" & DEBUG_S_RHDTES_FSM0_STATE;
      when "00001" =>
        DEBUG_STATE <= "0000" & DEBUG_S_RHDTES_FSM1_STATE;
      when "00010" =>
        DEBUG_STATE <= "00000" & DEBUG_S_RHD2ZL_FSM0_STATE;
      when "00011" =>
        DEBUG_STATE <= "00000" & DEBUG_S_RHDARR_FSM0_STATE;
      when "00100" =>
        DEBUG_STATE <= "0000" & DEBUG_S_DACOL0_FSM0_STATE;
      when "00101" =>
        DEBUG_STATE <= "0000" & DEBUG_S_DACOL1_FSM0_STATE;
      when "00110" =>
        DEBUG_STATE <= "0000" & DEBUG_S_DACOL2_FSM0_STATE;
      when "00111" =>
        DEBUG_STATE <= "0000" & DEBUG_S_DACOL3_FSM0_STATE;
      when "01000" =>
        DEBUG_STATE <= "0000" & DEBUG_S_DACOL4_FSM0_STATE;
      when "01001" =>
        DEBUG_STATE <= "0000" & DEBUG_S_DACOL5_FSM0_STATE;
      when "01010" =>
        DEBUG_STATE <= "0000" & DEBUG_S_DACOL6_FSM0_STATE;
      when "01011" =>
        DEBUG_STATE <= "0000" & DEBUG_S_DACOL7_FSM0_STATE;
      when "01100" =>
        DEBUG_STATE <= "0000" & DEBUG_S_COMREC_FSM0_STATE;
      when "01101" =>
        DEBUG_STATE <= "000" & DEBUG_S_ZARINI_FSM0_STATE;
      when "01110" =>
        DEBUG_STATE <= "00000" & DEBUG_S_ORGANI_FSM0_STATE;
      when "01111" =>
        DEBUG_STATE <= "000" & DEBUG_S_ZARSPI_FSM0_STATE;
      when "10000" =>
        DEBUG_STATE <= DEBUG_P_DAACBL_1;
      when "10001" =>
        DEBUG_STATE <= DEBUG_P_DAACBL_2;
      when "11111" =>
        DEBUG_STATE <= "111111111";
      when others =>
        DEBUG_STATE <= "111111111";
      end case;
  end process DEBUG_STATE_PROCESS;


  DEBUG_CLOCK_PROCESS : process (DEBUG_SELECTOR_CLOCK, DEBUG_CLOCK_ORIGINAL,
  DEBUG_CLOCK_HALF, DEBUG_CLOCK_4MHz)
  begin
    case DEBUG_SELECTOR_CLOCK is
      when "00" =>
        DEBUG_CLOCK <= DEBUG_CLOCK_ORIGINAL;
      when "01" =>
        DEBUG_CLOCK <= DEBUG_CLOCK_HALF;
      when "10" =>
        DEBUG_CLOCK <= DEBUG_CLOCK_4MHz;
      when "11" =>
        DEBUG_CLOCK <= '0';
      when others =>
        DEBUG_CLOCK <= '0';
      end case;
  end process DEBUG_CLOCK_PROCESS;


  ResetPocess : process (Clk,Reset)
  begin
    if Reset = '0' then
        ResetCounter <= (others => '0');
        Reset_NOT <= '1';
    elsif rising_edge(Clk) then
          if ResetCounter = WaitForReset then
            Reset_NOT <= '0';
            ResetCounter <= ResetCounter;
          else
            Reset_NOT <= '1';
            ResetCounter <= ResetCounter + 1;
          end if;
    end if;
  end process ResetPocess;

MYImplant : entity work.IMPLANT_TOP
  Port MAP (
    SPI_ZL_CS => SPI_ZL_CS,
    SPI_SDI => SPI_SDI,
    SPI_SDO => SPI_SDO,
    SPI_CLK => SPI_CLK,
    RHD_MOSI => RHD_MOSI,
    RHD_MISO => RHD_MISO,
    RHD_Clk	=> RHD_Clk,
    RHD_CS => RHD_CS,
    ERROR_ALL_OUT => open,
    DEBUG_S_RHDTES_FSM0_STATE => DEBUG_S_RHDTES_FSM0_STATE,
    DEBUG_S_RHDTES_FSM1_STATE => DEBUG_S_RHDTES_FSM1_STATE,
    DEBUG_S_RHD2ZL_FSM0_STATE => DEBUG_S_RHD2ZL_FSM0_STATE,
    DEBUG_S_RHDARR_FSM0_STATE => DEBUG_S_RHDARR_FSM0_STATE,
    DEBUG_S_DACOL0_FSM0_STATE => DEBUG_S_DACOL0_FSM0_STATE,
    DEBUG_S_DACOL1_FSM0_STATE => DEBUG_S_DACOL1_FSM0_STATE,
    DEBUG_S_DACOL2_FSM0_STATE => DEBUG_S_DACOL2_FSM0_STATE,
    DEBUG_S_DACOL3_FSM0_STATE => DEBUG_S_DACOL3_FSM0_STATE,
    DEBUG_S_DACOL4_FSM0_STATE => DEBUG_S_DACOL4_FSM0_STATE,
    DEBUG_S_DACOL5_FSM0_STATE => DEBUG_S_DACOL5_FSM0_STATE,
    DEBUG_S_DACOL6_FSM0_STATE => DEBUG_S_DACOL6_FSM0_STATE,
    DEBUG_S_DACOL7_FSM0_STATE => DEBUG_S_DACOL7_FSM0_STATE,
    DEBUG_CLOCK_ORIGINAL => DEBUG_CLOCK_ORIGINAL,
    DEBUG_CLOCK_HALF => DEBUG_CLOCK_HALF,
    DEBUG_CLOCK_4MHz => DEBUG_CLOCK_4MHz,
    DEBUG_S_COMREC_FSM0_STATE => DEBUG_S_COMREC_FSM0_STATE,
    DEBUG_S_ZARINI_FSM0_STATE => DEBUG_S_ZARINI_FSM0_STATE,
    DEBUG_S_ORGANI_FSM0_STATE => DEBUG_S_ORGANI_FSM0_STATE,
    DEBUG_S_ZARSPI_FSM0_STATE => DEBUG_S_ZARSPI_FSM0_STATE,
    DEBUG_P_DAACBL_1 => DEBUG_P_DAACBL_1,
    DEBUG_P_DAACBL_2 => DEBUG_P_DAACBL_2,
    Reset => Reset_NOT,
    Clk_FAST => Clk

  );

end architecture IGLOO_TOP_ARCH;
