-- Written by David Rotermund (davrot@neuro.uni-bremen.de)
-- University of Bremen, Institute for theoretical physiscs
-- (Use Atom.IO editor for correct indents)

-- The RHD test generator...

-- Version 0.0 (23.07.14) : Original source code was written
-- Version 0.1 (03.07.15) : Added debug info, reformating with Atom.IO

library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity RHD_TEST_MODULATOR is
  Port (
    -- For the data stream
    RHD_MOSI_OUTSIDE      :  in STD_LOGIC;
    RHD_MISO_OUTSIDE      : out STD_LOGIC;
    RHD_Clk_OUTSIDE       :  in STD_LOGIC;
    RHD_CS_OUTSIDE        :  in STD_LOGIC;

    -- The whole test thing fails with the IGLOO FPGA
    -- Thus we need to work on Clk instead of on RHD_Clk_OUTSIDE
    TEST_INFO             :  in STD_LOGIC;

    -- DEBUG SECTION
    DEBUG_M_FSM0_STATE    : out std_logic_vector(4 downto 0);
    DEBUG_M_FSM1_STATE    : out std_logic_vector(4 downto 0);

    Clk                   :  in STD_LOGIC;
    Reset                 :  in STD_LOGIC
  );
end RHD_TEST_MODULATOR;

architecture Behavioral of RHD_TEST_MODULATOR is

  type STATE_Type is (
    SM_Bit00,
    SM_Bit01,
    SM_Bit02,
    SM_Bit03,
    SM_Bit04,
    SM_Bit05,
    SM_Bit06,
    SM_Bit07,
    SM_Bit08,
    SM_Bit09,
    SM_Bit10,
    SM_Bit11,
    SM_Bit12,
    SM_Bit13,
    SM_Bit14,
    SM_Bit15
  );
  signal SPIMACHINE_STATE_I : STATE_TYPE;
  signal SPIMACHINE_STATE_O : STATE_TYPE;

  signal DATA_IN            : std_logic_vector(15 downto 0) := (others => '0');
  signal DATA_HOLD          : std_logic_vector(15 downto 0) := (others => '0');
  signal DATA_OUT           : std_logic_vector(15 downto 0) := (others => '0');

  signal DATA_OUT_COPY      : std_logic_vector(15 downto 0) := (others => '0');
  signal RHD_CS_OUTSIDE_LAST: std_logic := '0';

begin

  IN_PROCESS : process (Reset,Clk)
  begin
    if Reset = '1' then
      SPIMACHINE_STATE_I <= SM_Bit15;
      DATA_HOLD <= (others => '0');
      DATA_IN <= (others => '0');

    elsif rising_edge(Clk) then
      -- Default:
      SPIMACHINE_STATE_I <= SM_Bit15;

      case SPIMACHINE_STATE_I is
        when SM_Bit15 =>
          if TEST_INFO = '0' then
            SPIMACHINE_STATE_I <= SM_Bit15;
          else
            SPIMACHINE_STATE_I <= SM_Bit14;
          end if;

        when SM_Bit14 =>
          SPIMACHINE_STATE_I <= SM_Bit13;

        when SM_Bit13 =>
          SPIMACHINE_STATE_I <= SM_Bit12;

        when SM_Bit12 =>
          SPIMACHINE_STATE_I <= SM_Bit11;

        when SM_Bit11 =>
          SPIMACHINE_STATE_I <= SM_Bit10;

        when SM_Bit10 =>
          SPIMACHINE_STATE_I <= SM_Bit09;

        when SM_Bit09 =>
          SPIMACHINE_STATE_I <= SM_Bit08;

        when SM_Bit08 =>
          SPIMACHINE_STATE_I <= SM_Bit07;

        when SM_Bit07 =>
          SPIMACHINE_STATE_I <= SM_Bit06;

        when SM_Bit06 =>
          SPIMACHINE_STATE_I <= SM_Bit05;

        when SM_Bit05 =>
          SPIMACHINE_STATE_I <= SM_Bit04;

        when SM_Bit04 =>
          SPIMACHINE_STATE_I <= SM_Bit03;

        when SM_Bit03 =>
          SPIMACHINE_STATE_I <= SM_Bit02;

        when SM_Bit02 =>
          SPIMACHINE_STATE_I <= SM_Bit01;

        when SM_Bit01 =>
          SPIMACHINE_STATE_I <= SM_Bit00;

        when SM_Bit00 =>
          SPIMACHINE_STATE_I <= SM_Bit15;

        when others =>
          SPIMACHINE_STATE_I <= SM_Bit15;
      end case;

      DATA_HOLD <= DATA_HOLD;
      DATA_IN <= DATA_IN;
      case SPIMACHINE_STATE_I is
        when SM_Bit15 =>
          DATA_IN(15) <= RHD_MOSI_OUTSIDE;

        when SM_Bit14 =>
          DATA_IN(14) <= RHD_MOSI_OUTSIDE;

        when SM_Bit13 =>
          DATA_IN(13) <= RHD_MOSI_OUTSIDE;

        when SM_Bit12 =>
          DATA_IN(12) <= RHD_MOSI_OUTSIDE;

        when SM_Bit11 =>
          DATA_IN(11) <= RHD_MOSI_OUTSIDE;

        when SM_Bit10 =>
          DATA_IN(10) <= RHD_MOSI_OUTSIDE;

        when SM_Bit09 =>
          DATA_IN( 9) <= RHD_MOSI_OUTSIDE;

        when SM_Bit08 =>
          DATA_IN( 8) <= RHD_MOSI_OUTSIDE;

        when SM_Bit07 =>
          DATA_IN( 7) <= RHD_MOSI_OUTSIDE;

        when SM_Bit06 =>
          DATA_IN( 6) <= RHD_MOSI_OUTSIDE;

        when SM_Bit05 =>
          DATA_IN( 5) <= RHD_MOSI_OUTSIDE;

        when SM_Bit04 =>
          DATA_IN( 4) <= RHD_MOSI_OUTSIDE;

        when SM_Bit03 =>
          DATA_IN( 3) <= RHD_MOSI_OUTSIDE;

        when SM_Bit02 =>
          DATA_IN( 2) <= RHD_MOSI_OUTSIDE;

        when SM_Bit01 =>
          DATA_IN( 1) <= RHD_MOSI_OUTSIDE;

        when SM_Bit00 =>
          DATA_HOLD <= DATA_IN(15 downto 1) & RHD_MOSI_OUTSIDE;

        when others =>
          null;
      end case;
    end if;
  end process IN_PROCESS;

  OUT_PROCESS : process (Reset,Clk)
  begin
    if Reset = '1' then
      SPIMACHINE_STATE_O <= SM_Bit15;

    elsif falling_edge(Clk) then
      -- Default:
      SPIMACHINE_STATE_O <= SM_Bit15;

      case SPIMACHINE_STATE_O is
        when SM_Bit15 =>
          if TEST_INFO = '0' then
            SPIMACHINE_STATE_O <= SM_Bit15;
          else
            SPIMACHINE_STATE_O <= SM_Bit14;
          end if;

        when SM_Bit14 =>
          SPIMACHINE_STATE_O <= SM_Bit13;

        when SM_Bit13 =>
          SPIMACHINE_STATE_O <= SM_Bit12;

        when SM_Bit12 =>
          SPIMACHINE_STATE_O <= SM_Bit11;

        when SM_Bit11 =>
          SPIMACHINE_STATE_O <= SM_Bit10;

        when SM_Bit10 =>
          SPIMACHINE_STATE_O <= SM_Bit09;

        when SM_Bit09 =>
          SPIMACHINE_STATE_O <= SM_Bit08;

        when SM_Bit08 =>
          SPIMACHINE_STATE_O <= SM_Bit07;

        when SM_Bit07 =>
          SPIMACHINE_STATE_O <= SM_Bit06;

        when SM_Bit06 =>
          SPIMACHINE_STATE_O <= SM_Bit05;

        when SM_Bit05 =>
          SPIMACHINE_STATE_O <= SM_Bit04;

        when SM_Bit04 =>
          SPIMACHINE_STATE_O <= SM_Bit03;

        when SM_Bit03 =>
          SPIMACHINE_STATE_O <= SM_Bit02;

        when SM_Bit02 =>
          SPIMACHINE_STATE_O <= SM_Bit01;

        when SM_Bit01 =>
          SPIMACHINE_STATE_O <= SM_Bit00;

        when SM_Bit00 =>
          SPIMACHINE_STATE_O <= SM_Bit15;

        when others =>
          SPIMACHINE_STATE_O <= SM_Bit15;
      end case;
    end if;
  end process OUT_PROCESS;

  -- Look at MOSI
  IO_PROCESS : process (SPIMACHINE_STATE_O,DATA_OUT_COPY)
  begin

    case SPIMACHINE_STATE_O is
      when SM_Bit15 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY(15);

      when SM_Bit14 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY(14);

      when SM_Bit13 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY(13);

      when SM_Bit12 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY(12);

      when SM_Bit11 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY(11);

      when SM_Bit10 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY(10);

      when SM_Bit09 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 9);

      when SM_Bit08 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 8);

      when SM_Bit07 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 7);

      when SM_Bit06 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 6);

      when SM_Bit05 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 5);

      when SM_Bit04 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 4);

      when SM_Bit03 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 3);

      when SM_Bit02 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 2);

      when SM_Bit01 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 1);

      when SM_Bit00 =>
        RHD_MISO_OUTSIDE <= DATA_OUT_COPY( 0);

      when others =>
        RHD_MISO_OUTSIDE <= '0';
    end case;
  end process IO_PROCESS;

  -- Look at CS rising
  CS_RISING_PROCESS : process (Reset,Clk)
  begin
    if Reset = '1' then
      DATA_OUT <= (others => '0');
      DATA_OUT_COPY <= (others => '0');
      RHD_CS_OUTSIDE_LAST <= '0';

    elsif rising_edge(Clk) then
      -- Default:
      DATA_OUT <= DATA_OUT;
      DATA_OUT_COPY <= DATA_OUT_COPY;

      RHD_CS_OUTSIDE_LAST <= RHD_CS_OUTSIDE;
      if (RHD_CS_OUTSIDE = '0') AND (RHD_CS_OUTSIDE_LAST = '1') then
        DATA_OUT <= DATA_HOLD;
        DATA_OUT_COPY <= DATA_OUT;
      else
        null;
      end if;

    end if;
  end process CS_RISING_PROCESS;


  -- DEBUGGING FSM
  DEBUG_PROCESS_I : process (SPIMACHINE_STATE_I)
  begin
    case SPIMACHINE_STATE_I is
      when SM_Bit00 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 0,5));
      when SM_Bit01 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 1,5));
      when SM_Bit02 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 2,5));
      when SM_Bit03 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 3,5));
      when SM_Bit04 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 4,5));
      when SM_Bit05 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 5,5));
      when SM_Bit06 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 6,5));
      when SM_Bit07 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 7,5));
      when SM_Bit08 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 8,5));
      when SM_Bit09 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned( 9,5));
      when SM_Bit10 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned(10,5));
      when SM_Bit11 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned(11,5));
      when SM_Bit12 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned(12,5));
      when SM_Bit13 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned(13,5));
      when SM_Bit14 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned(14,5));
      when SM_Bit15 =>
        DEBUG_M_FSM0_STATE <= std_logic_vector(to_unsigned(15,5));
      when others =>
        DEBUG_M_FSM0_STATE <= "11111";
      end case;
  end process DEBUG_PROCESS_I;

  DEBUG_PROCESS_O : process (SPIMACHINE_STATE_O)
  begin
    case SPIMACHINE_STATE_O is
      when SM_Bit00 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 0,5));
      when SM_Bit01 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 1,5));
      when SM_Bit02 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 2,5));
      when SM_Bit03 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 3,5));
      when SM_Bit04 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 4,5));
      when SM_Bit05 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 5,5));
      when SM_Bit06 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 6,5));
      when SM_Bit07 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 7,5));
      when SM_Bit08 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 8,5));
      when SM_Bit09 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned( 9,5));
      when SM_Bit10 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned(10,5));
      when SM_Bit11 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned(11,5));
      when SM_Bit12 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned(12,5));
      when SM_Bit13 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned(13,5));
      when SM_Bit14 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned(14,5));
      when SM_Bit15 =>
        DEBUG_M_FSM1_STATE <= std_logic_vector(to_unsigned(15,5));
      when others =>
        DEBUG_M_FSM1_STATE <= "11111";
      end case;
  end process DEBUG_PROCESS_O;

end Behavioral;
